<?php

/**
 * The frontend-specific functionality of the plugin.
 *
 * @link       https://www.myagileprivacy.com/
 * @since      1.0.12
 *
 * @package    MyAgilePrivacy
 * @subpackage MyAgilePrivacy/frontend
 */

/**
 * The frontend-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the frontend-specific stylesheet and JavaScript.
 *
 * @package    MyAgilePrivacy
 * @subpackage MyAgilePrivacy/frontend
 * @author     https://www.myagileprivacy.com/
 */
class MyAgilePrivacyFrontend {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.12
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.12
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;
	public $plugin_obj;

	/**
	 * Scan vars
	 *
	 * @since    1.0.14
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $scan_mode;
	private $cookie_post_id;
	private $scan_config;
	private $scan_output;
	private $scan_log;
	private $scan_done;
	private $saved_settings;
	private $saved_post;
	private $head_script;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.12
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $plugin_obj )
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->plugin_obj = $plugin_obj;

		//blocked on elementor
		if( !is_admin() &&
			//elementor
			!isset( $_GET['elementor-preview'] ) &&
			//divi
			( !isset( $_GET['et_fb'] ) || $_GET['et_fb'] != 1 ) )
		{
			//[myagileprivacy_cookie_accept] shortcode
			add_shortcode( 'myagileprivacy_cookie_accept', array( $this, 'myagileprivacy_cookie_accept_button' ));

			//[myagileprivacy_cookie_reject] shortcode
			add_shortcode( 'myagileprivacy_cookie_reject', array( $this, 'myagileprivacy_cookie_reject_button' ));

			//[myagileprivacy_cookie_customize] shortcode
			add_shortcode( 'myagileprivacy_cookie_customize', array( $this, 'myagileprivacy_cookie_customize_button' ));

			//[myagileprivacy_extra_info] shortcode
			add_shortcode( 'myagileprivacy_extra_info', array( $this, 'myagileprivacy_extra_info' ));

			//[myagileprivacy_fixed_text] shortcode
			add_shortcode( 'myagileprivacy_fixed_text', array( $this, 'myagileprivacy_fixed_text' ));

			//[myagileprivacy_link] shortcode
			add_shortcode( 'myagileprivacy_link', array( $this, 'myagileprivacy_link' ));

			//[myagileprivacy_showconsent] shortcde
			add_shortcode( 'myagileprivacy_showconsent', array( $this, 'myagileprivacy_showconsent' ));

			//[myagileprivacy_blocked_content_notification] shortcode
			add_shortcode( 'myagileprivacy_blocked_content_notification', array( $this, 'myagileprivacy_blocked_content_notification' ));
		}
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 * Rewritten func to ignore messy mofile
	 * @since    1.3.9
	 * @access   private
	 */
	private function my_load_plugin_textdomain( $domain, $deprecated = false, $plugin_rel_path = false )
	{

		if( isset( $GLOBALS['wp_version'] ) && version_compare( $GLOBALS['wp_version'], '5.0', '<' ) )
		{
			$locale = apply_filters( 'plugin_locale', is_admin() ? get_user_locale() : get_locale(), $domain );
		}
		else
		{
			$locale = apply_filters( 'plugin_locale', determine_locale(), $domain );
		}

		$mofile = $domain . '-' . $locale . '.mo';

		/*
		// Try to load from the languages directory first.
		if ( load_textdomain( $domain, WP_LANG_DIR . '/plugins/' . $mofile ) ) {
			return true;
		}
		*/

		if ( false !== $plugin_rel_path ) {
			$path = WP_PLUGIN_DIR . '/' . trim( $plugin_rel_path, '/' );
		} elseif ( false !== $deprecated ) {
			_deprecated_argument( __FUNCTION__, '2.7.0' );
			$path = ABSPATH . trim( $deprecated, '/' );
		} else {
			$path = WP_PLUGIN_DIR;
		}

		return load_textdomain( $domain, $path . '/' . $mofile );
	}


	/**
	 * Function for checking if multilang is enabled
	 *
	 * @since    1.3.7
	 * @access   private
	 */
	function check_if_multilang_enabled()
	{
		global $locale;
		global $sitepress;

		$is_wpml_enabled = false;
		$is_polylang_enabled = false;
		$with_multilang = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( function_exists( 'icl_object_id' ) && $sitepress )
			{
				$is_wpml_enabled = true;
				$with_multilang = true;
			}

			if( defined( 'POLYLANG_FILE' ) )
			{
				$is_polylang_enabled = true;
				$with_multilang = true;
			}
		}

		return $with_multilang;
	}


	/**
	 * Function for checking if polylang is enabled
	 *
	 * @since    1.3.7
	 * @access   private
	 */
	function check_if_polylang_enabled()
	{
		global $locale;
		global $sitepress;

		$is_polylang_enabled = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( defined( 'POLYLANG_FILE' ) )
			{
				$is_polylang_enabled = true;
			}
		}

		return $is_polylang_enabled;
	}


	/**
	 * Function for checking if wpml is enabled
	 *
	 * @since    1.3.7
	 * @access   private
	 */
	function check_if_wpml_enabled()
	{
		global $locale;
		global $sitepress;

		$is_wpml_enabled = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( function_exists( 'icl_object_id' ) && $sitepress )
			{
				$is_wpml_enabled = true;
			}
		}

		return $is_wpml_enabled;
	}


	/**
	 * Function for reading if I have to reload translations
	 *
	 * @since    1.3.7
	 * @access   private
	 */
	private function check_if_to_reload_translation()
	{
		$settings = MyAgilePrivacy::get_settings();

		global $locale;
		global $sitepress;
		$is_wpml_enabled = false;
		$is_polylang_enabled = false;
		$with_multilang = false;
		$current_lang_sdt = null;
		$multilang_locale = null;
		$reload_translation = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( function_exists( 'icl_object_id' ) && $sitepress )
			{
				$is_wpml_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = $sitepress->get_default_language();
				$wpml_current_lang = ICL_LANGUAGE_CODE;

				$current_lang_sdt = $wpml_current_lang;

				$multilang_locale = get_locale();
			}

			if( defined( 'POLYLANG_FILE' ) )
			{
				$is_polylang_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = pll_default_language();
				$polylang_current_lang = pll_current_language();

				$current_lang_sdt = $polylang_current_lang;

				$multilang_locale = get_locale();
			}
		}

		if( $current_lang_sdt )
		{
			if( strpos( $settings['default_locale'], $current_lang_sdt ) !== false  )
			{
				//no action
				$reload_translation = false;
			}
			else
			{
				$reload_translation = true;
			}
		}
		else
		{
			//no action
			$reload_translation = false;
		}

		return $reload_translation;
	}


	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * @since    1.0.12
	 * @access   private
	 */
	private function set_locale()
	{
		$multilang_enabled = $this->check_if_multilang_enabled();

		/**/
		global $locale;

		$the_settings = MyAgilePrivacy::get_settings();

		if( $multilang_enabled == false && isset( $the_settings['default_locale'] ) )
		{
			$old_locale = $locale;
			$locale = $the_settings['default_locale'];
		}

		if( $multilang_enabled )
		{
			unload_textdomain( MAP_PLUGIN_SLUG );
		}

		$loaded = $this->my_load_plugin_textdomain(
			MAP_PLUGIN_SLUG,
			false,
			dirname( dirname( plugin_basename( __FILE__ ) ) ) . '/lang/'
		);

		if( $multilang_enabled == false && isset( $the_settings['default_locale'] ) )
		{
			$locale = $old_locale;
		}
	}

    /**
     * cookie accept button shortcode
	 * @since    1.0.12
	 * @access   public
     */
    public function myagileprivacy_cookie_accept_button( $atts )
    {
        $defaults = MyAgilePrivacy::get_default_settings();
        $settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );

        $reload_translation = $this->check_if_to_reload_translation();

        extract( shortcode_atts(array(
            'margin' => '',
        ), $atts ) );
        $margin_style = $margin!="" ? ' margin:'.$margin.'; ' : '';

        $the_text = stripslashes( $settings['button_accept_text'] );

        if( $reload_translation )
        {
        	$the_text = __('Accept', 'myagileprivacy');
        }

        $color_style = $settings['button_accept_link_color'] != "" ? ' color:'.$settings['button_accept_link_color'].'; ' : '';
        $background_style = $settings['button_accept_button_color'] != "" ? ' background-color:'.$settings['button_accept_button_color'].'; ' : '';

        $class = ' class="map-button map-button-style map-accept-button"';

		// colore icona = colore testo
		$icon_background_style = $settings['button_reject_link_color'] != "" ? ' background-color:'.$settings['button_reject_link_color'].'; ' : '';

        $link_tag = '<a role="button" data-map_action="accept" id="map-accept-button"'. $class.' style="'.$margin_style.'; '.$color_style.'; '.$background_style.';">';
		$link_tag .= '<span style="display:block; '.$icon_background_style.';"></span>';
		$link_tag .= $the_text . '</a>';

        return $link_tag;
    }

    /**
     * cookie reject button shortcode
	 * @since    1.0.12
	 * @access   public
     */
    public function myagileprivacy_cookie_reject_button( $atts )
    {
        $defaults = MyAgilePrivacy::get_default_settings();
        $settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );

        $reload_translation = $this->check_if_to_reload_translation();

        extract( shortcode_atts(array(
            'margin' => '',
        ), $atts ) );
        $margin_style = $margin!="" ? ' margin:'.$margin.'; ' : '';

        $the_text = stripslashes( $settings['button_reject_text'] );

        if( $reload_translation )
        {
        	$the_text = __('Refuse', 'myagileprivacy');
        }

        $color_style = $settings['button_reject_link_color'] != "" ? ' color:'.$settings['button_reject_link_color'].'; ' : '';
        $background_style = $settings['button_reject_button_color'] != "" ? ' background-color:'.$settings['button_reject_button_color'].'; ' : '';

        $class = ' class="map-button map-button-style map-reject-button"';

		// colore icona = colore testo
		$icon_background_style = $settings['button_reject_link_color'] != "" ? ' background-color:'.$settings['button_reject_link_color'].'; ' : '';

        $link_tag = '<a role="button" data-map_action="reject" id="map-reject-button"'. $class.' style="'.$margin_style.'; '.$color_style.'; '.$background_style.';">';
		$link_tag .= '<span style="display:block; '.$icon_background_style.';"></span>';
		$link_tag .= $the_text . '</a>';

        return $link_tag;
    }


    /**
     * cookie customize button shortcode
	 * @since    1.0.12
	 * @access   public
     */
    public function myagileprivacy_cookie_customize_button( $atts )
    {
        $defaults = MyAgilePrivacy::get_default_settings();
        $settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );

		$reload_translation = $this->check_if_to_reload_translation();

        extract( shortcode_atts(array(
            'margin' => '',
        ), $atts ) );
        $margin_style = $margin!="" ? ' margin:'.$margin.'; ' : '';

        $the_text = stripslashes( $settings['button_customize_text'] );

        if( $reload_translation )
        {
        	$the_text = __('Customize', 'myagileprivacy');
        }

        extract( shortcode_atts(array(
            'margin' => '',
        ), $atts ) );
        $margin_style = $margin!="" ? ' margin:'.$margin.'; ' : '';

        $defaults = MyAgilePrivacy::get_default_settings();
        $settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );

        $color_style = $settings['button_customize_link_color'] != "" ? ' color:'.$settings['button_customize_link_color'].'; ' : '';
        $background_style = $settings['button_customize_button_color'] != "" ? ' background-color:'.$settings['button_customize_button_color'].'; ' : '';

        $class = ' class="map-button map-button-style map-customize-button"';

		// colore icona = colore testo
		$icon_background_style = $settings['button_reject_link_color'] != "" ? ' background-color:'.$settings['button_reject_link_color'].'; ' : '';

        $link_tag = '<a role="button" data-map_action="customize" id="map-customize-button"'. $class.' style="'.$margin_style.'; '.$color_style.'; '.$background_style.';">';
		$link_tag .= '<span style="display:block; '.$icon_background_style.';"></span>';
		$link_tag .= $the_text . '</a>';

        return $link_tag;
    }


    /**
     * myagileprivacy_extra_info shortcode
	 * @since    1.3.12
	 * @access   public
     */
    public function myagileprivacy_extra_info( $atts )
    {
        $defaults = MyAgilePrivacy::get_default_settings();
        $settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );

		$reload_translation = $this->check_if_to_reload_translation();

        extract( shortcode_atts(array(
            'margin' => '',
        ), $atts ) );
        $margin_style = $margin!="" ? ' margin:'.$margin.'; ' : '';


 		$output_text = "";

		if (!function_exists('is_plugin_active')) {
			include_once(ABSPATH . 'wp-admin/includes/plugin.php');
		}

		if( is_plugin_active( 'myagilepixel/myagilepixel.php') )
		{
			$found_items = array();

			if( defined('MAPX_my_agile_pixel_ga_on') )
			{
				$found_items[] = __('Google Analytics in version 4 (GA4)', 'myagileprivacy');
			}

			if( defined('MAPX_my_agile_pixel_fbq_on') )
			{
				$found_items[] = __('Facebook Remarketing', 'myagileprivacy');
			}

			if( defined('MAPX_my_agile_pixel_tiktok_on') )
			{
				$found_items[] = __('TikTok Pixel', 'myagileprivacy');
			}

			if( count( $found_items ) > 0 )
			{
				$found_items_string = implode( ', ', $found_items );

				$output_text = '<br>'.__('In addition, this site installs', 'myagileprivacy').' '.$found_items_string.' '.__('with anonymous data transmission via proxy.', 'myagileprivacy')."<br>".__('By giving your consent, the data will be sent anonymously, thus protecting your privacy.', 'myagileprivacy');
				;
			}
		}

        return $output_text;
    }

    /**
     * fixed_text shortcode
	 * @since    1.0.12
	 * @access   public
     */
    public function myagileprivacy_fixed_text( $atts )
    {
        extract( shortcode_atts(array(
            'text' => '',
            'lang' => '',
        ), $atts ) );

        $defaults = MyAgilePrivacy::get_default_settings();
        $settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );
        $rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

        $remove_dpo_text = true;

        if( isset( $rconfig ) && $rconfig['allow_dpo_edit'] &&
        	isset( $settings['display_dpo'] ) && $settings['display_dpo'] == 1 &&
        	isset( $settings['dpo_email'] ) && $settings['dpo_email'] )
        {
        	$remove_dpo_text = false;
        }

        $cookies_categories_data = $this->get_cookie_categories_description( false );
        $cookie_list_html = "";
        foreach ($cookies_categories_data as $k => $v)
        {
        	foreach ($v as $key => $value)
        	{
        		$cookie_list_html .= "<p><b>".$value['post_data']->post_title."</b><br>";
        		$cookie_list_html .= $value['post_data']->post_content."</p>";
        	}
        }

        if( !$text )
        {
        	$text = 'cookie_policy';
        }

        if( $lang )
        {
			global $locale;
			global $sitepress;
			$is_wpml_enabled = false;
			$is_polylang_enabled = false;
			$with_multilang = false;

			if( MAP_MULTILANG_SUPPORT )
			{
				if( function_exists( 'icl_object_id' ) && $sitepress )
				{
					$is_wpml_enabled = true;
					$with_multilang = true;

					$multilang_default_lang = $sitepress->get_default_language();
					$wpml_current_lang = ICL_LANGUAGE_CODE;
					$language_list = icl_get_languages();
					$language_list_codes = array();

					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $multilang_default_lang );
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $wpml_current_lang );

					foreach( $language_list as $k => $v )
					{
						if( isset( $v['code'] ) )
						{
							$language_list_codes[] = $v['code'];
						}
						elseif( isset( $v['language_code'] ) )
						{
							$language_list_codes[] = $v['language_code'];
						}
					}
				}

				if( defined( 'POLYLANG_FILE' ) )
				{
					$is_polylang_enabled = true;
					$with_multilang = true;

					$multilang_default_lang = pll_default_language();
					$language_list_codes = pll_languages_list();

					$polylang_current_lang = pll_current_language();

					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
				}
			}
        }

		if( $lang && $is_wpml_enabled )
		{
			$sitepress->switch_lang( $lang );
		}

		//check if exists
		$cc_args = array(
		    'posts_per_page'   => 	-1,
		    'post_type'        =>	MAP_POST_TYPE_POLICY,
		    'meta_key'         => 	'_map_remote_id',
		    'meta_value'       => 	$text
		);

		if( $lang && $is_polylang_enabled )
		{
			$cc_args['lang'] = $lang;
		}

		wp_reset_query();
		$cc_query = new WP_Query( $cc_args );

		//wpml language reset
		if( $lang && $is_wpml_enabled )
		{
			$sitepress->switch_lang( $wpml_current_lang );
		}

		if ( $cc_query->have_posts() )
		{
		    foreach ( $cc_query->get_posts() as $p )
		    {
		        $the_id = $p->ID;

		        //if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $the_id );

		        $content = get_post_field( 'post_content', $the_id );

		        if( $text == 'personal_data_policy' )
		        {
			        $map_option1_ack = get_post_meta( $the_id, '_map_option1_ack', true );
			        $map_option1_on = get_post_meta( $the_id, '_map_option1_on', true );

			        if( $map_option1_ack == 1 && $map_option1_on == 1 )
			        {
			        	//no action
			        }
			        else
			        {
			        	$content = preg_replace( '#(<li class="map_marketing_consent">).*?(</li>)#', '' , $content );
			        	$content = preg_replace( '#(<p class="map_marketing_consent">).*?(</p>)#', '' , $content );
			        }
		        }

		        if( $remove_dpo_text )
		        {
		        	$content = preg_replace( '#(<p class="map_dpo_text">).*?(</p>)#', '' , $content );
		        }
		        else
		        {
		        	$content = str_replace( 'MAP_DPO_MAIL', $settings['dpo_email'], $content );
		        }

				//block_the_content_filter mode
				if( isset( $rconfig ) &&
					isset( $rconfig['block_the_content_filter'] ) &&
					$rconfig['block_the_content_filter'] == 1 )
		        {
		        	//no action
		        }
		        else
		        {
					//add paragraphs to text
					$content = apply_filters( 'the_content', $content );
		        }

		        $website_name = get_site_url();

		        if( isset( $settings['website_name'] ) && $settings['website_name'] != '' )
		        {
		        	$website_name = $settings['website_name'];

			        //block_the_content_filter mode
			        if( isset( $rconfig ) &&
			        	isset( $rconfig['block_the_content_filter'] ) &&
			        	$rconfig['block_the_content_filter'] == 1 )
			        {
			        	//no action
			        }
			        else
			        {
				        //add paragraphs to text
				        $website_name = apply_filters( 'the_content', $website_name );
			        }
		        }

		        $content = str_replace( '[website_name]', $website_name, $content );
		        $content = str_replace( '[identity_name]', $settings['identity_name'], $content );
		        $content = str_replace( '[identity_address]', $settings['identity_address'], $content );
		        $content = str_replace( '[identity_email]', $settings['identity_email'], $content );
		        $content = str_replace( '[cookie_list]', $cookie_list_html, $content );

		        if( $settings['wrap_shortcodes'] )
		        {
		        	$content = '<div id="myagileprivacy_text_wrapper" class="myagileprivacy_text_wrapper">'.$content.'</div>';
		        }

		        return $content;
		    }
		}

		wp_reset_query();
    }


    /**
     * link shortcode
	 * @since    1.0.12
	 * @access   public
     */
    public function myagileprivacy_link( $atts )
    {
        extract( shortcode_atts(array(
            'value' => '',
            'text' => '',
        ), $atts ) );

        $defaults = MyAgilePrivacy::get_default_settings();
        $settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );

        if( $value == 'cookie_policy' )
        {
			$is_cookie_policy_url = $settings['is_cookie_policy_url'];
			$cookie_policy_url = $settings['cookie_policy_url'];
			$cookie_policy_page = $settings['cookie_policy_page'];

			if( !$text )
			{
				$text = "Cookie Policy";
			}

			if( $is_cookie_policy_url && $cookie_policy_url )
			{
				$the_url = $cookie_policy_url;
			}

			if( !$is_cookie_policy_url && $cookie_policy_page )
			{
				if( defined( 'POLYLANG_FILE' ) )
				{
					$the_url = get_permalink( pll_get_post( $cookie_policy_page ) );
				}
				else
				{
					$the_url = get_permalink( $cookie_policy_page );
				}
			}

			if( isset( $the_url ) )
			{
				$html = '<a href="'.$the_url.'" target="blank">'.$text.'</a>';

				return $html;
			}

        }

        if( $value == 'personal_data_policy' )
        {
			$is_personal_data_policy_url = $settings['is_personal_data_policy_url'];
			$personal_data_policy_url = $settings['personal_data_policy_url'];
			$personal_data_policy_page = $settings['personal_data_policy_page'];

			if( !$text )
			{
				$text = "Personal Data Policy";
			}

			if( $is_personal_data_policy_url && $personal_data_policy_url )
			{
				$the_url = $personal_data_policy_url;
			}

			if( !$is_personal_data_policy_url && $personal_data_policy_page )
			{
				if( defined( 'POLYLANG_FILE' ) )
				{
					$the_url = get_permalink( pll_get_post( $personal_data_policy_page ) );
				}
				else
				{
					$the_url = get_permalink( $personal_data_policy_page );
				}
			}

			if( isset( $the_url ) )
			{
				$html = '<a href="'.$the_url.'" target="blank">'.$text.'</a>';

				return $html;
			}
        }
        return;
    }

    /**
     * showconsent shortcode
	 * @since    1.2.8
	 * @access   public
     */
    public function myagileprivacy_showconsent( $atts )
    {
		$html = '<a href="#" class="showConsentAgain">'.__('Manage consent', 'myagileprivacy').'</a>';

		return $html;
    }


    /**
     * link shortcode
	 * @since    1.2.8
	 * @access   public
     */
    public function myagileprivacy_blocked_content_notification( $atts )
    {
        extract( shortcode_atts(array(
            'api_key' => '',
            'text' => '',
        ), $atts ) );

        if( $api_key )
        {
        	if( !$text )
        	{
				switch( $api_key )
				{
					case 'google_recaptcha':
						$text = __('Your cookie choices may not allow the form to be submitted. You can review your choices by <a href="#" class="showConsentAgain">clicking here</a>.', 'myagileprivacy');
						break;
				}
        	}

			$html = '<p class="map_custom_notify map_api_key_'.$api_key.'">'.$text.'</p>';

			return wp_kses( $html, MyAgilePrivacy::allowed_html_tags() );;
        }

        return;
    }

	/**
	 * inject html code in the site footer
	 * @since    1.0.12
	 * @access   public
	 */
	public function inject_html_code()
	{
		$the_options = MyAgilePrivacy::get_settings();

		if( !is_admin() &&
			//elementor
			!isset( $_GET['elementor-preview'] ) &&
			//divi
			( !isset( $_GET['et_fb'] ) || $_GET['et_fb'] != 1 ) &&
			$the_options['is_on'] == true )
		{
			$reload_translation = $this->check_if_to_reload_translation();

			$is_polylang_enabled = $this->check_if_polylang_enabled();

			$is_wpml_enabled = $this->check_if_wpml_enabled();

			require_once plugin_dir_path(__FILE__) . 'views/my-agile-privacy-notify.php';
		}
	}


	/**
	 * Register the stylesheets for the frontend area.
	 * @since    1.0.12
	 * @access   public
	 */
	public function enqueue_styles()
	{
		$the_options = MyAgilePrivacy::get_settings();
		if( $the_options['is_on'] == true )
		{
			wp_enqueue_style( $this->plugin_name, plugin_dir_url(__FILE__) . 'css/my-agile-privacy-frontend.css', array(), $this->version, 'all' );
		}
	}


	/**
	 * Register the js for the frontend area.
	 * @since    1.0.12
	 * @access   public
	 */
	public function enqueue_scripts()
	{
		$the_options = MyAgilePrivacy::get_settings();

		if( $the_options['is_on'] == true )
		{
			$load_type = 'std';

			$rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

			$skip = MyAgilePrivacy::check_buffer_skip_conditions();

			if( $skip == 'true_due_to_post' )
			{
				wp_enqueue_script( $this->plugin_name, plugin_dir_url(__FILE__) . 'js/empty.js', array(), $this->version, false );

				$blocks_to_inject = $this->get_head_script_string( true );

				foreach( $blocks_to_inject['inline'] as $k => $elem )
				{
					wp_add_inline_script( $this->plugin_name, $elem );
				}

				foreach( $blocks_to_inject['enqueue'] as $k => $elem )
				{
					wp_enqueue_script( $this->plugin_name.'_enqueue_'.$k, $elem, array('jquery'), $this->version, false );
				}
			}

			if( !( wp_script_is( 'jquery', 'registered') ||
				wp_script_is( 'jquery-core', 'registered') ||
				wp_script_is( 'jquery-core-js', 'registered')  ) )
			{
				$load_type = 'reinject_jquery';
			}

			if( isset( $rconfig ) &&
				isset( $rconfig['force_standard_jquery_inject'] ) &&
				$rconfig['force_standard_jquery_inject'] == 1 )
			{
				$load_type = 'std';
			}

			if( isset( $rconfig ) &&
				isset( $rconfig['force_reinject_jquery'] ) &&
				$rconfig['force_reinject_jquery'] == 1 )
			{
				$load_type = 'reinject_jquery';
			}

			if( $the_options['client_force_reinject_jquery'] )
			{
				$load_type = 'reinject_jquery';
			}

			if( $load_type == 'std' )
			{
				wp_enqueue_script( $this->plugin_name, plugin_dir_url(__FILE__) . 'js/my-agile-privacy-frontend.js', array('jquery'), $this->version, false );
			}

			if( $load_type == 'reinject_jquery' )
			{
				wp_enqueue_script( 'jquery-restored', 'https://code.jquery.com/jquery-3.6.0.min.js', false, '3.6.0' );

				wp_enqueue_script( $this->plugin_name, plugin_dir_url(__FILE__) . 'js/my-agile-privacy-frontend.js', array(), $this->version, false );
			}

			wp_localize_script( $this->plugin_name, 'map_cookiebar_settings', MyAgilePrivacy::get_json_settings() );

			wp_localize_script( $this->plugin_name, 'map_ajax',
				array(
					'ajax_url' 					=> admin_url( 'admin-ajax.php' ),
					'security' 					=> wp_create_nonce( 'map_js_shield_callback' ),
					'force_js_learning_mode' 	=> $rconfig['force_js_learning_mode']
			) );
		}
	}

	/**
	* function for registering custom post type
	 * @since    1.0.12
	 * @access   public
	*/
	public function register_custom_post_type()
	{
		$labels = array(
			'name'					=> __('My Agile Privacy', 'myagileprivacy'),
			'all_items'             => __('Cookie List', 'myagileprivacy'),
			'singular_name'			=> __('Cookie', 'myagileprivacy'),
			'add_new'				=> __('Add New Cookie', 'myagileprivacy'),
			'add_new_item'			=> __('Add New Cookie', 'myagileprivacy'),
			'edit_item'				=> __('Edit Cookie', 'myagileprivacy'),
			'new_item'				=> __('New Cookie', 'myagileprivacy'),
			'view_item'				=> __('View Cookie', 'myagileprivacy'),
			'search_items'			=> __('Search Cookies', 'myagileprivacy'),
			'not_found'				=> __('Nothing found', 'myagileprivacy'),
			'not_found_in_trash'	=> __('Nothing found in Trash', 'myagileprivacy'),
			'parent_item_colon'		=> ''
		);
		$args = array(
			'labels'				=> $labels,
			'public'				=> false,
			'publicly_queryable'	=> false,
			'exclude_from_search'	=> true,
			'show_ui'				=> true,
			'query_var'				=> true,
			'rewrite'				=> true,
			'menu_icon' 			=> plugin_dir_url(__DIR__).'/admin/img/logo.png',
			'capabilities' => array(
				'publish_posts' => 'manage_options',
				'edit_posts' => 'manage_options',
				'edit_others_posts' => 'manage_options',
				'delete_posts' => 'manage_options',
				'delete_others_posts' => 'manage_options',
				'read_private_posts' => 'manage_options',
				'edit_post' => 'manage_options',
				'delete_post' => 'manage_options',
				'read_post' => 'manage_options',
			),
			'hierarchical'			=> false,
			'menu_position' 		=> 5,
			'supports'				=> array('title', 'editor'),
			'can_export'			=> false,
		);
		register_post_type( MAP_POST_TYPE_COOKIES, $args );


	    register_post_status( '__expired', array(
	        'label'                     => _x( 'Expired', MAP_POST_TYPE_COOKIES ),
	        'public'                    => false,
	        'exclude_from_search'       => true,
	        'show_in_admin_all_list'    => true,
	        'show_in_admin_status_list' => true,
	        'label_count'               => _n_noop( 'Expired <span class="count">(%s)</span>', 'Expired <span class="count">(%s)</span>' ),
	    ) );


		$labels = array(
			'name'					=> __('Policies', 'myagileprivacy'),
			'all_items'             => __('Policies List', 'myagileprivacy'),
			'singular_name'			=> __('Policy', 'myagileprivacy'),
			'add_new'				=> __('Add New Policy', 'myagileprivacy'),
			'add_new_item'			=> __('Add New Policy', 'myagileprivacy'),
			'edit_item'				=> __('Edit Policy', 'myagileprivacy'),
			'new_item'				=> __('New Policy', 'myagileprivacy'),
			'view_item'				=> __('View Policy', 'myagileprivacy'),
			'search_items'			=> __('Search Policy', 'myagileprivacy'),
			'not_found'				=> __('Nothing found', 'myagileprivacy'),
			'not_found_in_trash'	=> __('Nothing found in Trash', 'myagileprivacy'),
			'parent_item_colon'		=> ''
		);
		$args = array(
			'labels'				=> $labels,
			'public'				=> false,
			'publicly_queryable'	=> false,
			'exclude_from_search'	=> true,
			'show_ui'				=> true,
			'query_var'				=> true,
			'rewrite'				=> true,
			'capabilities' => array(
				'publish_posts' => 'manage_options',
				'edit_posts' => 'manage_options',
				'edit_others_posts' => 'manage_options',
				'delete_posts' => 'do_not_allow',
				'delete_others_posts' => 'manage_options',
				'read_private_posts' => 'manage_options',
				'edit_post' => 'manage_options',
				'delete_post' => 'do_not_allow',
				'read_post' => 'manage_options',
				'create_posts' => 'do_not_allow',
			),
			'hierarchical'			=> false,
			'menu_position'			=> null,
			'supports'				=> array('title', 'editor'),
			'can_export'			=> false,
		);
		register_post_type( MAP_POST_TYPE_POLICY, $args );

	    register_post_status( '__expired', array(
	        'label'                     => _x( 'Expired', MAP_POST_TYPE_POLICY ),
	        'public'                    => false,
	        'exclude_from_search'       => true,
	        'show_in_admin_all_list'    => true,
	        'show_in_admin_status_list' => true,
	        'label_count'               => _n_noop( 'Expired <span class="count">(%s)</span>', 'Expired <span class="count">(%s)</span>' ),
	    ) );

		remove_post_type_support( MAP_POST_TYPE_POLICY, 'title' );

		//blocked on elementor
		if( !is_admin() &&
			//elementor
			!isset( $_GET['elementor-preview'] ) &&
			//divi
			( !isset( $_GET['et_fb'] ) || $_GET['et_fb'] != 1 ) )
		{
			$this->set_locale();
		}
	}

	/**
	* function for getting cookie list displayed
	 * @since    1.0.12
	 * @access   public
	*/
	public function get_cookie_categories_description( $get_only_detectable=false )
	{
        $defaults = MyAgilePrivacy::get_default_settings();
        $settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );

		$rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

		if( $get_only_detectable )
		{
			$post_status = array( 'draft' );
		}
		else
		{
			$post_status = array( 'publish' );
		}

		$cookie_categories = array(
			'necessary'			=>	array(),
			'not_necessary'		=>	array(),
		);

		$cc_args = array(
		    'posts_per_page'   => 	-1,
		    'post_type'        =>	MAP_POST_TYPE_COOKIES,
		    'post_status' 	   => 	$post_status,
		);
		wp_reset_query();
		$cc_query = new WP_Query( $cc_args );

		$i = 0;

		if ( $cc_query->have_posts() )
		{
		    foreach ( $cc_query->get_posts() as $p )
		    {
		        $the_id = $p->ID;

		        $post_data = get_post( $the_id );
		        $post_title = get_the_title( $the_id );

		        $the_post_content = stripslashes( $post_data->post_content );

		        $elem = array(
		        	'post_meta' 	=> 	get_post_meta( $the_id ),
		        	'post_data' 	=> 	$post_data,
		        	'post_title'	=>	$post_title,
		        	'post_content' 	=> 	$the_post_content,
		        	'remote_id'		=>	null,
		        );

		        if( isset( $elem['post_meta']['_map_remote_id'][0] ) )
		        {
		        	$elem['remote_id'] = $elem['post_meta']['_map_remote_id'][0];
		        }
		        else
		        {
		        	$elem['remote_id'] = '_c'.$i;
		        	$i++;
		        }

		        $cookie_categories[ $elem['post_meta']['_map_is_necessary'][0] ][] = $elem;
		    }
		}

		wp_reset_query();

		return $cookie_categories;
	}

    /**
     * Plugin auto update
	 * @since    1.1.9
	 * @access   public
     */
	public function auto_update_plugins ( $update, $item )
	{
		$rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

		$plugins = array ( MAP_PLUGIN_SLUG );
		if ( in_array( $item->slug, $plugins ) ) {

			if( isset( $rconfig ) &&
				isset( $rconfig['disable_plugin_autoupdate'] ) &&
				$rconfig['disable_plugin_autoupdate'] == 1 )
			{
				// use default settings
				return $update;
			}
			else
			{
				// update plugin
				return true;
			}

		} else {
			// use default settings
			return $update;
		}
	}


	/**
	 * buffer start
	 * @since    1.0.14
	 * @access   public
	*/
	public function map_buffer_start()
	{
		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'map_buffer_start' );

		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $_REQUEST );

		$skip = MyAgilePrivacy::check_buffer_skip_conditions();

	    if( $skip != 'true' )
	    {
	    	global $post;

	    	$this->saved_post = $post;

			$defaults = MyAgilePrivacy::get_default_settings();
			$settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );

			$rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

			$this->saved_settings = $settings;

			$this->scan_mode = $settings['scan_mode'];

			if( !isset( $this->scan_mode ) || $this->scan_mode == 'turned_off' )
			{
				$skip = true;

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'turned_off' );
			}
		}

		if( $skip != 'true' )
		{
			$post_status = array( 'publish' );

			if( $this->scan_mode == 'learning_mode' )
			{
				$post_status = array( 'publish', 'draft' );
			}

			$parse_config = array();

			if( isset( $this->saved_settings['parse_config'] ) )
			{
				$parse_config = json_decode( $this->saved_settings['parse_config'], true );
			}

			global $locale;
			global $sitepress;
			$is_wpml_enabled = false;
			$is_polylang_enabled = false;
			$with_multilang = false;

			if( MAP_MULTILANG_SUPPORT )
			{
				if( function_exists( 'icl_object_id' ) && $sitepress )
				{
					$is_wpml_enabled = true;
					$with_multilang = true;

					$multilang_default_lang = $sitepress->get_default_language();
					$wpml_current_lang = ICL_LANGUAGE_CODE;
					$language_list = icl_get_languages();
					$language_list_codes = array();

					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $multilang_default_lang );
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $wpml_current_lang );

					foreach( $language_list as $k => $v )
					{
						if( isset( $v['code'] ) )
						{
							$language_list_codes[] = $v['code'];
						}
						elseif( isset( $v['language_code'] ) )
						{
							$language_list_codes[] = $v['language_code'];
						}
					}
				}

				if( defined( 'POLYLANG_FILE' ) )
				{
					$is_polylang_enabled = true;
					$with_multilang = true;

					$multilang_default_lang = pll_default_language();
					$language_list_codes = pll_languages_list();

					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
				}
			}

			//check if exists
			if( $is_wpml_enabled )
			{
				$sitepress->switch_lang( $multilang_default_lang );
			}

			$cc_args = array(
			    'posts_per_page'   	=> 	-1,
			    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
			    'post_status' 		=> 	$post_status,
			);

			if( $is_polylang_enabled )
			{
				$cc_args['lang'] = $multilang_default_lang;
			}

			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $cc_args );
			wp_reset_query();
			$cc_query = new WP_Query( $cc_args );

			//wpml language reset
			if( $is_wpml_enabled )
			{
				$sitepress->switch_lang( $wpml_current_lang );
			}

			$key_to_enable = array();
			$key_to_not_block = array();
			$friendly_name_map = array();

			if ( $cc_query->have_posts() )
			{
				foreach ( $cc_query->get_posts() as $p )
				{
					$main_post_id = $p->ID;
		        	$main_post_title = get_the_title( $p->ID );

		        	$the_key = get_post_meta( $main_post_id, '_map_api_key', true );
		        	$is_necessary = get_post_meta( $main_post_id, '_map_is_necessary', true );

		        	if( $the_key )
		        	{
						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $the_key );
						$key_to_enable[] = $the_key;

						$friendly_name_map[ $the_key ] = $main_post_title;

						if( $is_necessary == 'necessary' )
						{
							$key_to_not_block[] = $the_key;
						}

						if( $this->scan_mode == 'learning_mode' )
						{
							$this->cookie_post_id[] = $main_post_id;
						}
		        	}
				}
			}

			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $key_to_enable );
			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $friendly_name_map );

		    foreach( $parse_config as $k => &$v )
		    {
		    	foreach( $v as $kk => &$vv )
		    	{
		    		if( in_array( $vv['key'], $key_to_enable ) )
		    		{
		    			$vv['active'] = true;

		    			if( isset( $friendly_name_map[ $vv['key'] ] ) )
		    			{
		    				$vv['friendly_name'] = $friendly_name_map[ $vv['key'] ];
		    			}

		    			if( in_array( $vv['key'], $key_to_not_block ) )
		    			{
		    				$vv['to_block'] = false;
		    			}
		    		}
		    	}
		    }

	    	$this->scan_config = $parse_config;

	    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $parse_config );

	    	wp_reset_query();

	    	if( !( isset( $rconfig ) &&
	    		isset( $rconfig['js_legacy_mode'] ) &&
	    		$rconfig['js_legacy_mode'] == 1 ) )
	    	{
	    		$this->head_script = $this->get_head_script_string( false );
	    	}

	        ob_start( array( $this, 'map_callback' ) );
	    }
	}

	/**
	 * buffer end
	 * @since    1.0.14
	 * @access   public
	*/
	public function map_buffer_end()
	{
	    if( ob_get_level() )
	    {
	        ob_end_flush();
	    }
	}

	/**
	 * get head script string
	 * @since    1.3.5
	 * @access   public
	*/
	public function get_head_script_string( $block_mode = false )
	{
		$defaults = MyAgilePrivacy::get_default_settings();
		$settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );
		$rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

    	//populating cookie_api_key_remote_id_map_active
    	$cookie_reset_timestamp = ( isset( $settings['cookie_reset_timestamp'] ) ) ? $settings['cookie_reset_timestamp'] : '';
    	$cookie_reset_timestamp_rich = $cookie_reset_timestamp;

    	if( $cookie_reset_timestamp_rich )
    	{
    		$cookie_reset_timestamp_rich = '_'.$cookie_reset_timestamp;
    	}

    	$cookies_categories_data = $this->get_cookie_categories_description( false );
    	$cookie_api_key_remote_id_map_active = array();
    	$cookie_api_key_friendly_name_map = array();
    	$cookie_api_key_not_to_block = array();

		foreach( $cookies_categories_data as $k => $v )
		{
			foreach( $v as $key => $value )
			{
				$the_post_title = $value['post_title'];
				$the_remote_id = $value['remote_id'];
				$the_post_api_key = isset( $value['post_meta']['_map_api_key'][0] ) ? $value['post_meta']['_map_api_key'][0] : null;
				$is_necessary = isset( $value['post_meta']['_map_is_necessary'][0] ) ? $value['post_meta']['_map_is_necessary'][0] : 'not-necessary';

				if( $the_remote_id && $the_post_api_key )
				{
					$cookie_api_key_remote_id_map_active[ $the_post_api_key ] = 'map_cookie_'.$the_remote_id.$cookie_reset_timestamp_rich;

					if( $is_necessary == 'necessary' )
					{
						$cookie_api_key_not_to_block[] = $the_post_api_key;
					}

					if( $the_post_title && $is_necessary != 'necessary' )
					{
						$cookie_api_key_friendly_name_map[ $the_post_api_key ] = $the_post_title;
					}
				}
			}
		}

		//populating cookie_api_key_remote_id_map_detectable
    	$cookies_categories_data_detectable = $this->get_cookie_categories_description( true );
    	$cookie_api_key_remote_id_map_detectable = array();

		foreach( $cookies_categories_data_detectable as $k => $v )
		{
			foreach( $v as $key => $value )
			{
				$the_post_title = $value['post_title'];
				$the_remote_id = $value['remote_id'];
				$the_post_api_key = isset( $value['post_meta']['_map_api_key'][0] ) ? $value['post_meta']['_map_api_key'][0] : null;

				if( $the_remote_id && $the_post_api_key )
				{
					$cookie_api_key_remote_id_map_detectable[ $the_post_api_key ] = 'map_cookie_'.$the_remote_id.$cookie_reset_timestamp_rich;

					if( $the_post_title )
					{
						$cookie_api_key_friendly_name_map[ $the_post_api_key ] = $the_post_title;
					}
				}
			}
		}

		$cookie_reset_timestamp = ( isset( $settings['cookie_reset_timestamp'] ) ) ? $settings['cookie_reset_timestamp'] : 'null';

		$enforce_youtube_privacy = ( isset( $settings['enforce_youtube_privacy'] ) ) ? intval( $settings['enforce_youtube_privacy'] ) : 0;

		$blocks = array(
			'inline' 	=> array(),
			'enqueue'	=> array(),
		);

		$base_config_script = 'var cookie_reset_timestamp='.$cookie_reset_timestamp.';var cookie_api_key_remote_id_map_active='.json_encode( $cookie_api_key_remote_id_map_active ).';var cookie_api_key_remote_id_map_detectable='.json_encode( $cookie_api_key_remote_id_map_detectable ).';var cookie_api_key_friendly_name_map='.json_encode( $cookie_api_key_friendly_name_map ).';var cookie_api_key_not_to_block='.json_encode( $cookie_api_key_not_to_block ).';var enforce_youtube_privacy='.$enforce_youtube_privacy.';';


		$blocks['inline'][] = $base_config_script;

		$start_config_script = '<script class="map_advanced_shield" type="text/javascript">'.$base_config_script.'</script>';

		$head_script = '';

    	if( $settings['pa'] == 1 )
    	{
    		if( $enforce_youtube_privacy )
    		{
    			$the_script = plugin_dir_url(__FILE__).'js/youtube_enforced_privacy.js';

    			$head_script .= '<script class="map_advanced_shield_youtube_enforced_privacy" type="text/javascript" src="'.$the_script.'"></script>';

    			$blocks['enqueue'][] = $the_script;


    		}

			if( MAP_DEV_MODE )
			{
				$the_script = plugin_dir_url(__FILE__).'../dev/dev.cookie-shield.js';

				$head_script .= '<script class="map_advanced_shield" type="text/javascript" src="'.$the_script.'"></script>';

				$blocks['enqueue'][] = $the_script;
			}
			else
			{
				$base_ref = "https://cdn.myagileprivacy.com/";

		        //js_shield_test_mode mode
		        if( isset( $rconfig ) &&
		        	isset( $rconfig['alt_cdn'] ) )
		    	{
		    		$base_ref = $rconfig['alt_cdn'];
		    	}

		        //js_shield_test_mode mode
		        if( isset( $rconfig ) &&
		        	isset( $rconfig['js_shield_test_mode'] ) &&
		        	$rconfig['js_shield_test_mode'] == 1 )
		        {
		        	$the_script = $base_ref.'test.cookie-shield.js';

		        	$head_script .= '<script class="map_advanced_shield" type="text/javascript" src="'.$the_script.'"></script>';

		        	$blocks['enqueue'][] = $the_script;
		        }
		        else
		        {
		        	$the_script = $base_ref.'cookie-shield.js';

					$head_script .= '<script class="map_advanced_shield" type="text/javascript" src="'.$the_script.'"></script>';

					$blocks['enqueue'][] = $the_script;
		        }
			}
    	}

    	if( $block_mode )
    	{
    		return $blocks;
    	}
    	else
    	{

    		return $start_config_script.$head_script;
    	}

	}

	/**
	 * head inject
	 * @since    1.3.2
	 * @access   public
	*/
	public function wp_head_inject()
	{
		$skip = MyAgilePrivacy::check_buffer_skip_conditions();

		if( $skip == 'false' )
		{
			echo $this->get_head_script_string( false );
		}
	}

	/**
	 * Internal functin for detected keys save
	 *
	 * @since    1.3.5
	 */
	private function internal_save_detected_keys( $key=null )
	{
		// Get options
		$the_options = MyAgilePrivacy::get_settings();

		if( isset( $key )  )
		{
			if( $key != $the_options['license_code'] )
			{
				if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'Wrong key' );

				return false;
			}
		}

   		if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'start internal_save_detected_keys' );

		// Get options
		$js_cookie_shield_detected_keys = explode( ',', get_option( MAP_PLUGIN_JS_DETECTED_FIELDS ) );

		if( !$js_cookie_shield_detected_keys )
		{
			$js_cookie_shield_detected_keys = array();
		}

		//$js_cookie_shield_detected_keys = array();

		$detectableKeys = explode( ',', $_POST['detectableKeys'] );

		foreach( $detectableKeys as $k => $v )
		{
			if( $v )
			{
				$js_cookie_shield_detected_keys[] = $v;
			}
		}

		update_option( MAP_PLUGIN_JS_DETECTED_FIELDS, implode( ',', array_unique( $js_cookie_shield_detected_keys ) ) );


		global $locale;
		global $sitepress;
		$is_wpml_enabled = false;
		$is_polylang_enabled = false;
		$with_multilang = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( function_exists( 'icl_object_id' ) && $sitepress )
			{
				$is_wpml_enabled = true;
				$is_polylang_enabled = true;

				$multilang_default_lang = $sitepress->get_default_language();
				$wpml_current_lang = ICL_LANGUAGE_CODE;
				$language_list = icl_get_languages();
				$language_list_codes = array();

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $multilang_default_lang );
				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $wpml_current_lang );

				foreach( $language_list as $k => $v )
				{
					if( isset( $v['code'] ) )
					{
						$language_list_codes[] = $v['code'];
					}
					elseif( isset( $v['language_code'] ) )
					{
						$language_list_codes[] = $v['language_code'];
					}
				}
			}

			if( defined( 'POLYLANG_FILE' ) )
			{
				$is_polylang_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = pll_default_language();
				$language_list_codes = pll_languages_list();

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
			}
		}

		$auto_activate_keys = array();

	    foreach( $js_cookie_shield_detected_keys as $k => $v )
	    {
	    	if( $v )
	    	{
	    		$auto_activate_keys[] = $v;
	    	}
	    }

	    foreach( $auto_activate_keys as $k => $v )
	    {
			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $v, true );
			if( $v )
			{
				if( $is_wpml_enabled )
				{
					$sitepress->switch_lang( $multilang_default_lang );
				}

				$cc_args = array(
				    'posts_per_page'   	=> 	-1,
				    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
				    'meta_key'         	=> 	'_map_api_key',
				    'meta_value'       	=> 	$v,
				    'post_status' 		=> 	array( 'draft', 'publish' )
				);

				if( $is_polylang_enabled )
				{
					$cc_args['lang'] = $multilang_default_lang;
				}

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $cc_args, true );
				wp_reset_query();
				$cc_query = new WP_Query( $cc_args );

				//wpml language reset
				if( $is_wpml_enabled )
				{
					$sitepress->switch_lang( $wpml_current_lang );
				}

				if ( $cc_query->have_posts() )
				{
					foreach ( $cc_query->get_posts() as $p )
					{
			        	$main_post_id = $p->ID;

			        	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $main_post_id, true );

						//update
						$my_post = array(
							'ID'           	=> 	$main_post_id,
							'post_status'	=>	'publish',
						);

						if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

						wp_update_post( $my_post );

						if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( "published ".$v , true );

						update_post_meta( $main_post_id, "_map_auto_detected", 1 );

					    if( $with_multilang )
					    {
					    	if( $is_wpml_enabled )
					    	{
								$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );

				    			$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_COOKIES );

								//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $t_post_id );
			    				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $translations );
					    	}

					    	if( $is_polylang_enabled )
					    	{
					    		$translations = pll_get_post_translations( $main_post_id );
					    	}

			    			if( $translations )
			    			{
			    				$to_update = array();

						    	foreach( $translations as $t_key => $t_value )
						    	{
						    		if( $t_key != $multilang_default_lang )
						    		{
						    			if( $is_wpml_enabled )
						    			{
						    				$to_update[ $t_key ] = $t_value->element_id;
						    			}

						    			if( $is_polylang_enabled )
						    			{
						    				$to_update[ $t_key ] = $t_value;
						    			}
						    		}
						    	}

						    	if( $to_update )
						    	{
						    		foreach( $to_update as $key_to_update => $to_update_post_id )
						    		{
						    			if( $to_update_post_id )
						    			{
											//update
											$my_post = array(
												'ID'           	=> 	$to_update_post_id,
												'post_status'	=>	'publish',
											);

											//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

									    	wp_update_post( $my_post );

									    	update_post_meta( $to_update_post_id, "_map_auto_detected", 1 );
						    			}
						    		}
						    	}
			    			}
					    }

					}
				}
				wp_reset_query();
			}
	    }

	    if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'end internal_save_detected_keys' );

	    return true;
	}

	/**
	 * Save remotely detected keys callback function
	 *
	 * @since    1.3.0
	 */
	public function map_remote_save_detected_keys_callback()
	{
		$success = false;

		// check form submit
	    if( isset( $_POST['key'] ) )
	   	{
	        $success = $this->internal_save_detected_keys( $_POST['key'] );
	   	}

        $answer = array(
        	'success'				=>	$success,
        );

        wp_send_json( $answer );

        wp_die();
	}

	/**
	 * Save detected keys callback function
	 *
	 * @since    1.3.0
	 */
	public function map_save_detected_keys_callback()
	{
		//check security param
		check_ajax_referer( 'map_js_shield_callback', 'security' );

		$success = false;

		// check form submit
	    if( isset( $_POST['action'] ) && $_POST['action'] == 'map_save_detected_keys' )
	   	{
	        $success = $this->internal_save_detected_keys();
	   	}

        $answer = array(
        	'success'				=>	$success,
        );

        wp_send_json( $answer );

        wp_die();
	}


	/**
	 * map_callback
	 * @since    1.0.14
	 * @access   public
	*/
	public function map_callback( $output )
	{
		$this->scan_log = "Scanner started.<br>";

		//if( MAP_DEBUGGER ) $this->scan_log .= "map_callback\n\n";

		//init config
		$parse_config = $this->scan_config;
		$scan_output = $parse_config;

		//init html parser
		$dom = shd_str_get_html( $output, true, true, false, false );

		if( is_object( $dom ) )
		{
			//search for scripts
			$scripts = $dom->find( 'script' );

			//if( MAP_DEBUGGER ) $this->scan_log .= print_r( $scripts, true );

			if( isset( $parse_config ) && isset( $parse_config['scripts_src_block'] ) )
			{
				foreach( $scripts as $k => $v )
				{
					//if( MAP_DEBUGGER ) $this->scan_log .= print_r( $scripts, true );

					//class
					$class = $v->class;

					//inline
					$innertext = $v->innertext;

					//src
					$src = $v->src;

					if( strpos( $class, 'my_agile_privacy_activate') === false && strpos( $class, 'my_agile_privacy_do_not_touch') === false )
					{
						if( !empty( $innertext ) )
						{
							$found = array_filter( $parse_config['scripts_src_block'],  function( $e, $the_key ) use ( $innertext ){

								if( !$e['plain_js'] ) return false;

								return ( strpos( $innertext, $e['plain_js'] ) !== false );

							}, ARRAY_FILTER_USE_BOTH );

							if( $found )
							{
								$action = array_values( $found )[0];
								$return_key = array_keys( $found )[0];

								if( isset( $action ) )
								{
									$detected_pattern = $action['plain_js'];
									$detected_key = $action['key'];

									$go_on = false;
									$no_action = false;
									$deactivated_by_setting = ( isset( $action['deactivated_by_setting'] ) ) ? $action['deactivated_by_setting'] : null;

									if( $deactivated_by_setting == null ||
											( isset( $deactivated_by_setting ) && $this->saved_settings[ $deactivated_by_setting ] == true ) )
									{
										$go_on = true;
									}

									if( isset( $action['only_on_compatibility_mode'] ) &&
										$action['only_on_compatibility_mode'] == true &&
										$this->saved_settings['scanner_compatibility_mode'] == false )
									{
										$no_action = true;
									}

									if( ( $action['active'] || $go_on ) && !$no_action )
									{
										if( $action['to_detect'] == 1 )
										{
											$this->scan_log .= "detected plain_js script $detected_key $detected_pattern<br>";

											$scan_output['scripts_src_block'][ $return_key ]['detected'] = 1;

											$this->scan_done = true;
										}

										if( $go_on )
										{
											if( $action['to_block'] == 1 )
											{
												if( !( isset( $action['silent_blocking'] ) && $action['silent_blocking'] == true ) )
												{
													if( MAP_DEBUGGER ) $this->scan_log .= "blocked plain_js script $detected_key $detected_pattern\n\n";
													$v->class = $v->class .' '.'my_agile_privacy_activate autoscan_mode map_inline_script_blocked map_blocked_content';
												}

												if( isset( $action['on_block_add_classes'] ) )
												{
													$v->class = $v->class.' '.$action['on_block_add_classes'];
												}

												$v->type = 'text/plain';

												$v->setAttribute( 'data-cookie-api-key', $detected_key );

												$v->setAttribute( 'data-friendly_name', $action['friendly_name'] );

												$scan_output['scripts_src_block'][ $return_key ]['blocked'] = 1;

												$this->scan_done = true;
											}
											else
											{
												$v->class = $v->class.' map_not_to_block';
											}
										}
									}
								}
								//if( MAP_DEBUGGER ) $this->scan_log .= .= print_r( array_values($found)[0], true )."\n";
							}
						}
						elseif( $src )
						{
							$found = array_filter( $parse_config['scripts_src_block'],  function( $e, $the_key ) use ( $src ){

								if( !$e['src'] ) return false;

								return ( strpos(  $src, $e['src'] ) !== false );

							}, ARRAY_FILTER_USE_BOTH );

							if( $found )
							{
								//if( MAP_DEBUGGER ) $this->scan_log .= print_r($found,true)."\n";

								$action = array_values( $found )[0];
								$return_key = array_keys( $found )[0];

								if( isset( $action ) )
								{
									$detected_pattern = $action['src'];
									$detected_key = $action['key'];

									$go_on = false;
									$no_action = false;
									$deactivated_by_setting = ( isset( $action['deactivated_by_setting'] ) ) ? $action['deactivated_by_setting'] : null;
									if( $deactivated_by_setting == null ||
											( isset( $deactivated_by_setting ) && $this->saved_settings[ $deactivated_by_setting ] == true ) )
									{
										$go_on = true;
									}

									if( isset( $action['only_on_compatibility_mode'] ) &&
										$action['only_on_compatibility_mode'] == true &&
										$this->saved_settings['scanner_compatibility_mode'] == false )
									{
										$no_action = true;
									}

									if( ( $action['active'] || $go_on ) && !$no_action )
									{
										if( $action['to_detect'] == 1 )
										{
											$this->scan_log .= "detected src script $detected_key $detected_pattern<br>";

											$scan_output['scripts_src_block'][ $return_key ]['detected'] = 1;

											$this->scan_done = true;
										}

										if( $go_on )
										{
											if( $action['to_block'] == 1 )
											{

												if( !( isset( $action['silent_blocking'] ) && $action['silent_blocking'] == true ) )
												{
													if( MAP_DEBUGGER ) $this->scan_log .= "blocked src script $detected_key $detected_pattern\n\n";
													$v->class = $v->class .' '.'my_agile_privacy_activate autoscan_mode map_src_script_blocked map_blocked_content';
												}

												if( isset( $action['on_block_add_classes'] ) )
												{
													$v->class = $v->class.' '.$action['on_block_add_classes'];
												}

												$v->unblocked_src = $v->src;
												$v->src = '';

												$v->setAttribute( 'data-cookie-api-key', $detected_key );
												$v->setAttribute( 'data-friendly_name', $action['friendly_name'] );

												$scan_output['scripts_src_block'][ $return_key ]['blocked'] = 1;

												$this->scan_done = true;
											}
											else
											{
												$v->class = $v->class.' map_not_to_block';
											}
										}
									}
								}
								//if( MAP_DEBUGGER ) $this->scan_log .= print_r( array_values($found)[0], true )."\n";
							}
						}
						else
						{
							//no action
						}
					}
				}
			}


			//search for iframes
			$iframes = $dom->find( 'iframe' );

			if( isset( $parse_config ) && isset( $parse_config['iframe_src_block'] ) )
			{
				foreach( $iframes as $k => $v )
				{
					//src
					$src = $v->src;

					if( strpos( $v->class, 'lazyload' ) !== false )
					{
						$src = $v->getAttribute( 'data-src' );
					}

					//if( MAP_DEBUGGER ) $this->scan_log .= print_r( $src, true )."\n";

					$found = array_filter( $parse_config['iframe_src_block'],  function( $e, $the_key ) use ( $src ){

						if( !$e['src'] ) return false;

						return ( strpos(  $src, $e['src'] ) !== false );

					}, ARRAY_FILTER_USE_BOTH );


					if( $found )
					{
						if( strpos( $v->class, 'lazyload' ) !== false )
						{
							$v->class = str_replace( 'lazyload', '', $v->class );
						}

						//if( MAP_DEBUGGER ) $this->scan_log .= print_r($found,true)."\n";

						$action = array_values( $found )[0];
						$return_key = array_keys( $found )[0];

						if( isset( $action ) )
						{
							$detected_pattern = $action['src'];
							$detected_key = $action['key'];

							$go_on = false;
							$no_action = false;
							$deactivated_by_setting = ( isset( $action['deactivated_by_setting'] ) ) ? $action['deactivated_by_setting'] : null;

							if( $deactivated_by_setting == null ||
									( isset( $deactivated_by_setting ) && $this->saved_settings[ $deactivated_by_setting ] == true ) )
							{
								$go_on = true;
							}

							if( isset( $action['only_on_compatibility_mode'] ) &&
								$action['only_on_compatibility_mode'] == true &&
								$this->saved_settings['scanner_compatibility_mode'] == false )
							{
								$no_action = true;
							}

							if( ( $action['active'] || $go_on ) && !$no_action )
							{
								if( $action['to_detect'] == 1 )
								{
									$this->scan_log .= "detected iframe $detected_key $detected_pattern<br>";

									$scan_output['iframe_src_block'][ $return_key ]['detected'] = 1;

									$this->scan_done = true;
								}

								if( $go_on )
								{
									if( $action['to_block'] == 1 )
									{
										if( MAP_DEBUGGER ) $this->scan_log .= "blocked iframe $detected_key $detected_pattern\n\n";

										$v->class = $v->class.' '.'my_agile_privacy_activate autoscan_mode iframe_src_blocked map_blocked_content';

										if( isset( $action['show_inline_notify'] ) )
										{
											$v->class = $v->class.' map_show_inline_notify';
										}

										if( isset( $action['on_block_add_classes'] ) )
										{
											$v->class = $v->class.' '.$action['on_block_add_classes'];
										}

										$v->unblocked_src = $src;
										$v->src = '';
										$v->setAttribute( 'data-cookie-api-key', $detected_key );
										$v->setAttribute( 'data-friendly_name', $action['friendly_name'] );

										$scan_output['iframe_src_block'][ $return_key ]['blocked'] = 1;

										$this->scan_done = true;
									}

									if( $action['to_fix'] == 1 )
									{
										if( MAP_DEBUGGER ) $this->scan_log .= "fixed $detected_key $detected_pattern\n\n";

										if( $action['to_fix_search'] && $action['to_fix_replace']  )
										{
											$v->class = $v->class.' '.'map_script_fixed';
											$v->original_src = $src;
											$v->src = str_replace( $action['to_fix_search'], $action['to_fix_replace'], $src );
										}

										if( $action['to_append_param'] )
										{
											$v->class = $v->class.' '.'map_script_fixed';
											$v->original_src = $src;

											if( strpos( $v->original_src, '?' ) !== false )
											{
												$v->src = $src.'&'.$action['to_append_param'];
											}
											else
											{
												$v->src = $src.'?'.$action['to_append_param'];
											}

											$v->src = str_replace( $action['to_fix_search'], $action['to_fix_replace'], $src );
										}

										$scan_output['iframe_src_block'][ $return_key ]['fixed'] = 1;

										$this->scan_done = true;
									}
								}

							}
						}
					}
				}
			}



			//search for links
			$links = $dom->find( 'link[rel="stylesheet"]' );

			if( isset( $parse_config ) && isset( $parse_config['css_href_block'] ) )
			{
				foreach( $links as $k => $v )
				{
					//href
					$href = $v->href;
					$class = $v->class;

					if( strpos( $class, 'my_agile_privacy_activate') === false && strpos( $class, 'my_agile_privacy_do_not_touch') === false )
					{
						//if( MAP_DEBUGGER ) $this->scan_log .= print_r( $href, true )."\n";

						$found = array_filter( $parse_config['css_href_block'],  function( $e, $the_key ) use ( $href ){

							if( !$e['href'] ) return false;

							return ( strpos(  $href, $e['href'] ) !== false );

						}, ARRAY_FILTER_USE_BOTH );

						if( $found )
						{
							//if( MAP_DEBUGGER ) $this->scan_log .= print_r($found,true)."\n";

							$action = array_values( $found )[0];
							$return_key = array_keys( $found )[0];

							if( isset( $action ) )
							{
								$detected_pattern = $action['href'];
								$detected_key = $action['key'];

								$go_on = false;
								$no_action = false;
								$deactivated_by_setting = ( isset( $action['deactivated_by_setting'] ) ) ? $action['deactivated_by_setting'] : null;
								if( $deactivated_by_setting == null ||
										( isset( $deactivated_by_setting ) && $this->saved_settings[ $deactivated_by_setting ] == true ) )
								{
									$go_on = true;
								}

								if( isset( $action['only_on_compatibility_mode'] ) &&
									$action['only_on_compatibility_mode'] == true &&
									$this->saved_settings['scanner_compatibility_mode'] == false )
								{
									$no_action = true;
								}

								if( ( $action['active'] || $go_on ) && !$no_action )
								{
									if( $action['to_detect'] == 1 )
									{
										$this->scan_log .= "detected link $detected_key $detected_pattern<br>";

										$scan_output['css_href_block'][ $return_key ]['detected'] = 1;

										$this->scan_done = true;
									}

									if( $go_on )
									{
										if( $action['to_block'] == 1 )
										{
											if( MAP_DEBUGGER ) $this->scan_log .= "blocked link $detected_key $detected_pattern\n\n";

											$v->class = $v->class.' '.'my_agile_privacy_activate autoscan_mode css_href_blocked map_blocked_content';

											if( isset( $action['on_block_add_classes'] ) )
											{
												$v->class = $v->class.' '.$action['on_block_add_classes'];
											}

											$v->unblocked_href = $v->href;
											$v->href = '';
											$v->setAttribute( 'data-cookie-api-key', $detected_key );
											$v->setAttribute( 'data-friendly_name', $action['friendly_name'] );

											$scan_output['css_href_block'][ $return_key ]['blocked'] = 1;

											$this->scan_done = true;
										}
										else
										{
											$v->class = $v->class.' map_not_to_block';
										}
									}
								}
							}
						}
					}
				}
			}


			//elementor wigets
			$elementor_widgets = $dom->find( '.elementor-widget' );

			if( isset( $parse_config ) && isset( $parse_config['elementor_widget_block'] ) )
			{
				foreach( $elementor_widgets as $k => $v )
				{
					//data-settings
					$data_settings = $v->getAttribute( 'data-settings' );

					$class = $v->class;

					if( strpos( $class, 'my_agile_privacy_do_not_touch') === false )
					{
						//if( MAP_DEBUGGER ) $this->scan_log .= print_r( $src, true )."\n";

						$found = array_filter( $parse_config['elementor_widget_block'],  function( $e, $the_key ) use ( $data_settings ){

							if( !$e['src'] ) return false;

							return ( strpos(  $data_settings, $e['src'] ) !== false );

						}, ARRAY_FILTER_USE_BOTH );

						if( $found )
						{
							//if( MAP_DEBUGGER ) $this->scan_log .= print_r($found,true)."\n";

							$action = array_values( $found )[0];
							$return_key = array_keys( $found )[0];

							if( isset( $action ) )
							{
								$detected_pattern = $action['src'];
								$detected_key = $action['key'];

								$go_on = false;
								$no_action = false;
								$deactivated_by_setting = ( isset( $action['deactivated_by_setting'] ) ) ? $action['deactivated_by_setting'] : null;

								if( $deactivated_by_setting == null ||
										( isset( $deactivated_by_setting ) && $this->saved_settings[ $deactivated_by_setting ] == true ) )
								{
									$go_on = true;
								}

								if( isset( $action['only_on_compatibility_mode'] ) &&
									$action['only_on_compatibility_mode'] == true &&
									$this->saved_settings['scanner_compatibility_mode'] == false )
								{
									$no_action = true;
								}

								if( ( $action['active'] || $go_on ) && !$no_action )
								{
									if( $action['to_detect'] == 1 )
									{
										$this->scan_log .= "detected elementor-widget $detected_key $detected_pattern<br>";

										$scan_output['elementor_widget_block'][ $return_key ]['detected'] = 1;

										$this->scan_done = true;
									}

									if( $go_on )
									{
										if( $action['to_fix'] == 1 )
										{
											if( MAP_DEBUGGER ) $this->scan_log .= "fixed elementor-widget $detected_pattern\n\n";

											if( $action['fix_mode'] == 'json_append' )
											{
												$v->class = $v->class.' '.'map_script_fixed';
												$v->original_data_settings = $data_settings;

												//if( MAP_DEBUGGER ) $this->scan_log .= html_entity_decode( $data_settings );

												$decoded_data_settings = json_decode( html_entity_decode( $data_settings ), true );

												$decoded_data_settings = array_merge( (array)$decoded_data_settings,(array)$action['json_add'] );

												//if( MAP_DEBUGGER ) $this->scan_log .= print_r( $decoded_data_settings, true );

												$v->setAttribute( 'data-settings', htmlentities( json_encode( $decoded_data_settings ) ) );
											}

											$scan_output['elementor_widget_block'][ $return_key ]['fixed'] = 1;

											$this->scan_done = true;
										}
									}
								}
							}
						}
					}

				}
			}


			//elementor specific fixings (lightbox youtube)
			$elementor_widgets = $dom->find( '.elementor-widget .elementor-image a' );

			foreach( $elementor_widgets as $k => $v )
			{
				//href
				$href = $v->href;
				$class = $v->class;

				if( strpos( $class, 'my_agile_privacy_do_not_touch') === false )
				{
					if( strpos( $href, '#elementor-action%3Aaction%3Dlightbox' ) !== false )
					{
						$url_decoded = urldecode( $href );
						$exploded = explode( '&', $url_decoded );

						$to_fix = str_replace('settings=','', $exploded[1] );

						$to_fix_decoded = base64_decode( $to_fix );

						$to_fix_decoded_decoded = json_decode( $to_fix_decoded, true );

						$to_fix_decoded_decoded['url'] = str_replace( 'www.youtube.com', 'www.youtube-nocookie.com', $to_fix_decoded_decoded['url'] );

						$to_fix_decoded_encoded = json_encode($to_fix_decoded_decoded );

						$to_fix_encoded = base64_encode( $to_fix_decoded_encoded );

						$url_fixed = urlencode( str_replace( $to_fix, $to_fix_encoded, $url_decoded ) );

						$v->href = $url_fixed;
						$v->setAttribute( 'original-href', $href );
					}
				}
			}

			//elementor-background-video-container
			$elementor_widgets = $dom->find( '.elementor-background-video-container' );
			/**/
			foreach( $elementor_widgets as $k => $v )
			{
				$class = $v->class;

				if( strpos( $class, 'my_agile_privacy_do_not_touch') === false )
				{
					//parent
					$parent = $v->parent();

					if( $parent )
					{
						$data_settings = $parent->getAttribute( 'data-settings' );

						$action_to_add = array( 'background_privacy_mode' => 'yes' );

						$decoded_data_settings = json_decode( html_entity_decode( $data_settings ), true );

						$decoded_data_settings = array_merge( (array)$decoded_data_settings,(array)$action_to_add );

						$parent->setAttribute( 'data-settings', htmlentities( json_encode( $decoded_data_settings ) ) );

						$parent->class = $parent->class.' '.'map_script_fixed';
					}
				}
			}
			/**/

			//avia specific fixings (youtube)
			$avia_widgets = $dom->find( '.av-video-tmpl' );

			foreach( $avia_widgets as $k => $v )
			{
				//inline
				$innertext = $v->innertext;
				$class = $v->class;

				if( strpos( $class, 'my_agile_privacy_do_not_touch') === false )
				{
					if( strpos( $innertext, 'www.youtube.com' ) !== false )
					{
						$innertext = str_replace( 'www.youtube.com', 'www.youtube-nocookie.com', $innertext );

						$v->__set( 'innertext', $innertext );
					}
				}

			}

			if( MAP_DEBUGGER ) $this->scan_log .= print_r( $parse_config['img_search_replace'], true )."\n";

			//search for images
			$images = $dom->find( 'img' );

			if( isset( $parse_config ) && isset( $parse_config['img_search_replace'] ) )
			{
				foreach( $images as $k => $v )
				{
					//src
					$src = $v->src;
					$srcset = $v->getAttribute( 'srcset' );

					if( strpos( $v->class, 'lazyload' ) !== false )
					{
						$src = $v->getAttribute( 'data-src' );
					}

					//if( MAP_DEBUGGER ) $this->scan_log .= print_r( $src, true )."\n";

					$found = array_filter( $parse_config['img_search_replace'],  function( $e, $the_key ) use ( $src ){

						if( !$e['src'] ) return false;

						return ( strpos(  $src, $e['src'] ) !== false );

					}, ARRAY_FILTER_USE_BOTH );


					$found_alt = array_filter( $parse_config['img_search_replace'],  function( $e, $the_key ) use ( $srcset ){

						if( !$e['src'] ) return false;

						return ( strpos(  $srcset, $e['src'] ) !== false );

					}, ARRAY_FILTER_USE_BOTH );


					if( $found || $found_alt )
					{
						if( strpos( $v->class, 'lazyload' ) !== false )
						{
							$v->class = str_replace( 'lazyload', '', $v->class );
						}

						if( MAP_DEBUGGER ) $this->scan_log .= print_r($found,true)."\n";

						$action = array_values( $found )[0];
						$return_key = array_keys( $found )[0];

						if( isset( $action ) )
						{
							$detected_pattern = $action['src'];
							$detected_key = $action['key'];

							$go_on = false;
							$no_action = false;
							$deactivated_by_setting = ( isset( $action['deactivated_by_setting'] ) ) ? $action['deactivated_by_setting'] : null;

							if( $deactivated_by_setting == null ||
									( isset( $deactivated_by_setting ) && $this->saved_settings[ $deactivated_by_setting ] == true ) )
							{
								$go_on = true;
							}

							if( isset( $action['only_on_compatibility_mode'] ) &&
								$action['only_on_compatibility_mode'] == true &&
								$this->saved_settings['scanner_compatibility_mode'] == false )
							{
								$no_action = true;
							}

							if( ( $action['active'] || $go_on ) && !$no_action )
							{
								if( $action['to_detect'] == 1 )
								{
									$this->scan_log .= "detected img $detected_key $detected_pattern<br>";

									$scan_output['img_search_replace'][ $return_key ]['detected'] = 1;

									$this->scan_done = true;
								}

								if( $go_on )
								{
									if( $action['to_block'] == 1 )
									{
										if( MAP_DEBUGGER ) $this->scan_log .= "blocked img $detected_key $detected_pattern\n\n";

										$v->class = $v->class.' '.'my_agile_privacy_activate autoscan_mode img_src_blocked map_blocked_content';

										if( isset( $action['show_inline_notify'] ) )
										{
											$v->class = $v->class.' map_show_inline_notify';
										}

										if( isset( $action['on_block_add_classes'] ) )
										{
											$v->class = $v->class.' '.$action['on_block_add_classes'];
										}

										$v->unblocked_src = $src;
										//blank image
										$v->src = 'data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==';
										$v->setAttribute( 'srcset', '' );
										$v->setAttribute( 'data-cookie-api-key', $detected_key );
										$v->setAttribute( 'data-friendly_name', $action['friendly_name'] );

										$scan_output['img_search_replace'][ $return_key ]['blocked'] = 1;

										$this->scan_done = true;
									}
								}
							}
						}
					}
				}
			}

		}

		//if( MAP_DEBUGGER ) $this->scan_log .= print_r( $this->saved_settings, true );
		//if( MAP_DEBUGGER ) $this->scan_log .= print_r( $parse_config, true );
		//if( MAP_DEBUGGER ) $this->scan_log .= print_r( $scan_output, true );
		$this->scan_output = $scan_output;
		//if( MAP_DEBUGGER ) $this->scan_log .= "map_callback_end\n\n";
		//if( MAP_DEBUGGER ) return $this->scan_log;

		//$output = $this->scan_log;

		$output_ori = $dom;

		if( $this->head_script && is_object( $dom ) )
		{
			$inject = $this->head_script;

			$head_object = $dom->find('head', 0);

			if( $head_object && is_object( $head_object ) )
			{
				$head_object->innertext = $inject.$head_object->innertext;
			}

		}

		$output = $dom;

		/*
		$output_ori = $output;
		$count = 1;
		$output = str_ireplace('<head>', '<head>'.$start_config_script.$this->head_script, $output, $count );
		*/

		if( !$output )
		{
			$output = $output_ori;
		}

		$js_cookie_shield_detected_keys = explode( ',', get_option( MAP_PLUGIN_JS_DETECTED_FIELDS ) );

		if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $js_cookie_shield_detected_keys );

		if( $this->scan_log && MAP_DEBUGGER ) MyAgilePrivacy::write_log( $js_cookie_shield_detected_keys );

	    if( $this->scan_done || count( $js_cookie_shield_detected_keys ) > 0 )
	    {
	    	//if( $this->scan_output && MAP_DEBUGGER ) MyAgilePrivacy::write_log( $this->scan_output, true );
	    	if( $this->scan_log && MAP_DEBUGGER ) MyAgilePrivacy::write_log( $this->scan_log );

	    	//auto activate
	    	if( $this->scan_mode == 'learning_mode' )
	    	{
	    		//if( $this->scan_log && MAP_DEBUGGER ) MyAgilePrivacy::write_log( $this->cookie_post_id );

				// Get options
				$the_options = MyAgilePrivacy::get_settings();

				global $locale;
				global $sitepress;
				$is_wpml_enabled = false;
				$is_polylang_enabled = false;
				$with_multilang = false;

				if( MAP_MULTILANG_SUPPORT )
				{
					if( function_exists( 'icl_object_id' ) && $sitepress )
					{
						$is_wpml_enabled = true;
						$is_polylang_enabled = true;

						$multilang_default_lang = $sitepress->get_default_language();
						$wpml_current_lang = ICL_LANGUAGE_CODE;
						$language_list = icl_get_languages();
						$language_list_codes = array();

						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $multilang_default_lang );
						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $wpml_current_lang );

						foreach( $language_list as $k => $v )
						{
							if( isset( $v['code'] ) )
							{
								$language_list_codes[] = $v['code'];
							}
							elseif( isset( $v['language_code'] ) )
							{
								$language_list_codes[] = $v['language_code'];
							}
						}
					}

					if( defined( 'POLYLANG_FILE' ) )
					{
						$is_polylang_enabled = true;
						$with_multilang = true;

						$multilang_default_lang = pll_default_language();
						$language_list_codes = pll_languages_list();

						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
					}
				}

				$auto_activate_keys = array();

			    foreach( $this->scan_output as $k => $v )
			    {
			    	foreach( $v as $kk => $vv )
			    	{
			    		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $vv, true );

			    		$key_to_look_for = $vv['key'];

			    		if( $vv['key'] && $vv['detected'] )
			    		{
			    			$auto_activate_keys[] = $vv['key'];
			    		}
			    	}
			    }

			    foreach( $js_cookie_shield_detected_keys as $k => $v )
			    {
			    	$this->scan_log .= "JS Cookie Shield has detected $v <br>";

			    	if( $v )
			    	{
			    		$auto_activate_keys[] = $v;
			    	}
			    }

			    foreach( $auto_activate_keys as $k => $v )
			    {
	    			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $v, true );
	    			if( $v )
	    			{
						if( $is_wpml_enabled )
						{
							$sitepress->switch_lang( $multilang_default_lang );
						}

						$cc_args = array(
						    'posts_per_page'   	=> 	-1,
						    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
						    'meta_key'         	=> 	'_map_api_key',
						    'meta_value'       	=> 	$v,
						    'post_status' 		=> 	array( 'draft', 'publish' )
						);

						if( $is_polylang_enabled )
						{
							$cc_args['lang'] = $multilang_default_lang;
						}

						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $cc_args, true );
						wp_reset_query();
						$cc_query = new WP_Query( $cc_args );

						//wpml language reset
						if( $is_wpml_enabled )
						{
							$sitepress->switch_lang( $wpml_current_lang );
						}

						if ( $cc_query->have_posts() )
						{
							foreach ( $cc_query->get_posts() as $p )
							{
					        	$main_post_id = $p->ID;

					        	if( in_array( $main_post_id, $this->cookie_post_id ) )
					        	{
									//update
									$my_post = array(
										'ID'           	=> 	$main_post_id,
										'post_status'	=>	'publish',
									);

									if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

									wp_update_post( $my_post );

									if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( "published ".$v , true );

									update_post_meta( $main_post_id, "_map_auto_detected", 1 );

								    if( $with_multilang )
								    {
								    	if( $is_wpml_enabled )
								    	{
											$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );

							    			$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_COOKIES );

											//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $t_post_id );
						    				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $translations );
								    	}

								    	if( $is_polylang_enabled )
								    	{
								    		$translations = pll_get_post_translations( $main_post_id );
								    	}

						    			if( $translations )
						    			{
						    				$to_update = array();

									    	foreach( $translations as $t_key => $t_value )
									    	{
									    		if( $t_key != $multilang_default_lang )
									    		{
									    			if( $is_wpml_enabled )
									    			{
									    				$to_update[ $t_key ] = $t_value->element_id;
									    			}

									    			if( $is_polylang_enabled )
									    			{
									    				$to_update[ $t_key ] = $t_value;
									    			}
									    		}
									    	}

									    	if( $to_update )
									    	{
									    		foreach( $to_update as $key_to_update => $to_update_post_id )
									    		{
									    			if( $to_update_post_id )
									    			{
														//update
														$my_post = array(
															'ID'           	=> 	$to_update_post_id,
															'post_status'	=>	'publish',
														);

														//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

												    	wp_update_post( $my_post );

												    	update_post_meta( $to_update_post_id, "_map_auto_detected", 1 );
									    			}
									    		}
									    	}
						    			}
								    }
					        	}
							}
						}
						wp_reset_query();
	    			}
			    }

			    update_option( MAP_PLUGIN_JS_DETECTED_FIELDS, null );

			    $this->scan_log .= "Scanner finished.<br>";
				$the_options['scanner_log'] = htmlentities( stripslashes( html_entity_decode( $the_options['scanner_log'] )."<br>".$this->scan_log ) );
	    	}
	    }

	    if( $this->saved_settings['scanner_compatibility_mode'] )
	    {
		    global $post;
		    setup_postdata( $this->saved_post );
	    }

	    wp_reset_query();

		return $output;
	}
}