<?php

/**
 * Core definitions
 * *
 * @link       https://www.myagileprivacy.com/
 * @since      1.0.12
 *
 * @package    MyAgilePrivacy
 * @subpackage MyAgilePrivacy/includes
 */

/**
 * Core plugin class.
 *
 *
 * @since      1.0.12
 * @package    MyAgilePrivacy
 * @subpackage MyAgilePrivacy/includes
 * @author     https://www.myagileprivacy.com/
 */
class MyAgilePrivacy {

	/**
	 * Unique identifier of this plugin.
	 *
	 * @since    1.0.12
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * Current version of the plugin.
	 *
	 * @since    1.0.12
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	//stored user options
	private static $stored_options = array();

	/**
	 * Core functionality of the plugin.
	 *
	 * It sets plugin name, plugin version.
	 * It loads dependencies, set the locale, and hoocks for admin and frontend area
	 *
	 * @since    1.0.12
	 */
	public function __construct()
	{
		$this->version = MAP_PLUGIN_VERSION;
		$this->plugin_name = MAP_PLUGIN_NAME;

		$this->load_classes_and_dependencies();
		$this->admin_hooks();
		$this->frontend_hooks();
	}

	/**
	 * Load the required dependencies.
	 *
	 * @since    1.0.12
	 * @access   private
	 */
	private function load_classes_and_dependencies()
	{
		/**
		 * The class for defining all actions that occur in the backend area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/my-agile-privacy-admin.php';

		/**
		 * The class for defining all the functionalities for the frontend part
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'frontend/my-agile-privacy-frontend.php';
	}

	/**
	 * Register all of the hooks related to the frontend part
	 *
	 * @since    1.0.12
	 * @access   private
	 */
	private function frontend_hooks()
	{
		$plugin_frontend = new MyAgilePrivacyFrontend( $this->plugin_name, $this->version, $this );

		/* Frontend styles*/
		add_action( 'wp_enqueue_scripts', array( $plugin_frontend, 'enqueue_styles' ), PHP_INT_MIN );

		/* Frontend scripts*/
		add_action( 'wp_enqueue_scripts', array( $plugin_frontend, 'enqueue_scripts' ), PHP_INT_MIN );

		/* Custom post type*/
		add_action( 'init', array( $plugin_frontend, 'register_custom_post_type' ) );

		/* wp_footer hook*/
		add_action( 'wp_footer', array( $plugin_frontend, 'inject_html_code' ) );

		$the_settings = self::get_settings();

		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $the_settings );

		if( $the_settings['pa'] == 1 && MAP_SCANNER )
		{
			/* admin callback actions */
			add_action( 'wp_ajax_nopriv_map_save_detected_keys', array( $plugin_frontend, 'map_save_detected_keys_callback' ) );
			add_action( 'wp_ajax_map_save_detected_keys', array( $plugin_frontend, 'map_save_detected_keys_callback' ) );
			add_action( 'wp_ajax_nopriv_map_remote_save_detected_keys', array( $plugin_frontend, 'map_remote_save_detected_keys_callback' ) );
			add_action( 'wp_ajax_map_remote_save_detected_keys', array( $plugin_frontend, 'map_remote_save_detected_keys_callback' ) );
		}

		$skip = $this::check_buffer_skip_conditions();

		if( $skip == 'false' && $the_settings['pa'] == 1 && MAP_SCANNER )
		{
			$rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

			if( $rconfig &&
				isset( $rconfig['js_legacy_mode'] ) &&
				$rconfig['js_legacy_mode'] == 1 )
			{
				add_action( 'wp_head', array( $plugin_frontend, 'wp_head_inject' ), PHP_INT_MIN );
			}

			/**
			 * The class for html parsing
			 */

			if( !class_exists( 'agile_simple_html_dom_node' ) )
			{
				require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/simple_html_dom.php';
			}

			if( $the_settings['scanner_compatibility_mode'] &&
				$the_settings['scanner_start_hook_prio'] &&
				$the_settings['scanner_end_hook_prio'] )
			{

				if( $the_settings['scanner_hook_type'] == 'template_redirect-shutdown' )
				{
					//customized settings
					add_action( 'template_redirect', array( $plugin_frontend, 'map_buffer_start' ), $the_settings['scanner_start_hook_prio'] );
					add_action( 'shutdown', array( $plugin_frontend, 'map_buffer_end' ), $the_settings['scanner_end_hook_prio'] );
				}
				elseif( $the_settings['scanner_hook_type'] == 'init-shutdown' )
				{
					//customized settings
					add_action( 'init', array( $plugin_frontend, 'map_buffer_start' ), $the_settings['scanner_start_hook_prio'] );
					add_action( 'shutdown', array( $plugin_frontend, 'map_buffer_end' ), $the_settings['scanner_end_hook_prio'] );
				}
				else
				{
					//customized settings
					add_action( 'init', array( $plugin_frontend, 'map_buffer_start' ), $the_settings['scanner_start_hook_prio'] );
					add_action( 'shutdown', array( $plugin_frontend, 'map_buffer_end' ), $the_settings['scanner_end_hook_prio'] );
				}
			}
			else
			{
				//standard settings
				add_action( 'init', array( $plugin_frontend, 'map_buffer_start' ) );
				add_action( 'shutdown', array( $plugin_frontend, 'map_buffer_end' ), -1000 );
			}
		}

		/*auto update*/
		add_filter( 'auto_update_plugin', array( $plugin_frontend, 'auto_update_plugins' ), 10, 2 );
	}

	/**
	 * Register all of the hooks related to the backend area
	 *
	 * @since    1.0.12
	 * @access   private
	 */
	private function admin_hooks()
	{
		$rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

		$plugin_admin = new MyAgilePrivacyAdmin( $this->plugin_name, $this->version, $this );

		//admin menu
		add_action( 'admin_menu', array( $plugin_admin, 'add_admin_pages' ), 11 );

		//cookie list dashboard
		add_action( 'views_edit-'.MAP_POST_TYPE_COOKIES, array( $plugin_admin, 'map_fix_view_links' ), 11 );

		//default sorting
		add_filter( 'pre_get_posts', array( $plugin_admin, 'map_order_post_type' ) );

		//admin init && metabox
		add_action( 'admin_init', array( $plugin_admin, 'admin_init_and_add_meta_box' ) );
		add_action( 'save_post_'.MAP_POST_TYPE_COOKIES, array( $plugin_admin, 'save_custom_metabox_cookies' ) );
		add_action( 'save_post_'.MAP_POST_TYPE_POLICY, array( $plugin_admin, 'save_custom_metabox_policies' ) );

		//cookies
		add_action( 'manage_edit-'.MAP_POST_TYPE_COOKIES.'_columns', array( $plugin_admin, 'manage_cookies_edit_columns' ) );
		add_action( 'manage_'.MAP_POST_TYPE_COOKIES.'_posts_custom_column', array( $plugin_admin, 'manage_cookies_posts_custom_columns' ) );

		//policies
		add_action( 'manage_edit-'.MAP_POST_TYPE_POLICY.'_columns', array( $plugin_admin, 'manage_policies_edit_columns' ) );
		add_action( 'manage_'.MAP_POST_TYPE_POLICY.'_posts_custom_column', array( $plugin_admin, 'manage_policies_posts_custom_columns' ) );

		//inline help
		add_action( 'admin_notices', array( $plugin_admin, 'inline_help_text_after_editor' ) );

		//admin callback actions
		add_action( 'wp_ajax_nopriv_update_admin_settings_form', array( $plugin_admin, 'update_admin_settings_form_callback' ) );
		add_action( 'wp_ajax_update_admin_settings_form', array( $plugin_admin, 'update_admin_settings_form_callback' ) );

		//admin post actions
		add_action( 'admin_post_backup_admin_settings_form', array( $plugin_admin, 'backup_admin_settings_form_callback' ) );

		add_action( 'admin_post_import_admin_settings_form', array( $plugin_admin, 'import_admin_settings_form_callback' ) );

		//generic admin styles
		add_action( 'admin_enqueue_scripts', array( $plugin_admin, 'enqueue_styles' ) );

		add_action( 'admin_enqueue_scripts', function()
		{
			wp_enqueue_style( 'admin-myagileprivacy', plugin_dir_url( __DIR__ ) ."admin/css/my-agile-privacy-admin.css", array(),$this->version, 'all' );
		});

		//generic admin scripts
		add_action( 'admin_enqueue_scripts', array( $plugin_admin, 'enqueue_scripts' ) );

		//admin styles for css styles on custom post edit
		add_action( 'load-post.php', array( $plugin_admin, 'custom_post_type_editor' ) );
		add_action( 'load-post-new.php', array( $plugin_admin, 'custom_post_type_editor' ) );

		//add settings links for the menu
		add_filter( 'plugin_action_links_'.plugin_basename( MAP_PLUGIN_FILENAME ), array( $plugin_admin, 'plugin_action_links') );

		//add cron scheduled functions

		if( !( isset( $rconfig ) &&
				isset( $rconfig['disable_cronjob'] ) &&
				$rconfig['disable_cronjob'] == 1 ) )
		{
			add_action( 'my_agile_privacy_do_cron_sync_once_day_hook', array( $plugin_admin, 'do_cron_sync' ) );

			//clean old daily event schedule if exists
			if ( wp_next_scheduled( 'my_agile_privacy_do_cron_sync_hook' ) ) {
				wp_clear_scheduled_hook('my_agile_privacy_do_cron_sync_hook');
			}

			//clean twice a day event schedule if exists
			if ( wp_next_scheduled( 'my_agile_privacy_do_cron_sync_twice_day_hook' ) ) {
				wp_clear_scheduled_hook('my_agile_privacy_do_cron_sync_twice_day_hook');
			}

			//schedule an action if it's not already scheduled
			if ( ! wp_next_scheduled( 'my_agile_privacy_do_cron_sync_once_day_hook' ) ) {
			    wp_schedule_event( time(), 'daily', 'my_agile_privacy_do_cron_sync_once_day_hook' );
			}
		}
		else
		{
			//clean old daily event schedule if exists
			if ( wp_next_scheduled( 'my_agile_privacy_do_cron_sync_hook' ) ) {
				wp_clear_scheduled_hook('my_agile_privacy_do_cron_sync_hook');
			}

			//clean twice a day event schedule if exists
			if ( wp_next_scheduled( 'my_agile_privacy_do_cron_sync_twice_day_hook' ) ) {
				wp_clear_scheduled_hook('my_agile_privacy_do_cron_sync_twice_day_hook');
			}

			//clean once a day event schedule if exists
			if ( wp_next_scheduled( 'my_agile_privacy_do_cron_sync_once_day_hook' ) ) {
				wp_clear_scheduled_hook('my_agile_privacy_do_cron_sync_once_day_hook');
			}

		}

		if( !( isset( $rconfig ) &&
				isset( $rconfig['disable_install_counter'] ) &&
				$rconfig['disable_install_counter'] == 1 ) )
		{
			add_action( 'my_agile_privacy_do_cron_sync_install_counter', array( $plugin_admin, 'do_cron_sync_install_counter' ) );

			//schedule an action if it's not already scheduled
			if ( ! wp_next_scheduled( 'my_agile_privacy_do_cron_sync_install_counter' ) ) {
			    wp_schedule_event( time(), 'daily', 'my_agile_privacy_do_cron_sync_install_counter' );
			}

			if( !get_option( MAP_PLUGIN_STATS ) )
			{
				wp_schedule_single_event( time() + 5, 'my_agile_privacy_do_cron_sync_install_counter' );
			}
		}

		/*
		if( !( isset( $rconfig ) &&
				isset( $rconfig['slow_site_mode'] ) &&
				$rconfig['slow_site_mode'] == 1 ) )
		*/
		if( ( isset( $rconfig ) &&
				isset( $rconfig['admin_footer_hook'] ) &&
				$rconfig['admin_footer_hook'] == 1 ) )
		{
			add_action( 'admin_footer', array( $plugin_admin, 'admin_footer' ) );
		}

		add_action( 'admin_footer', array( $plugin_admin, 'admin_auto_enable_cookie' ) );

		if( defined( 'POLYLANG_FILE' ) )
		{
			//polylang
			add_filter( 'pll_get_post_types', array( $plugin_admin, 'add_cpt_to_pll' ), 10, 2 );
		}
	}

	/**
	 * Get current settings.
	 * @since    1.0.12
	 * @access   public
	 */
	public static function get_settings()
	{
		$settings = self::get_default_settings();
		self::$stored_options = get_option( MAP_PLUGIN_SETTINGS_FIELD );
		if(!empty(self::$stored_options))
		{
			foreach(self::$stored_options as $key => $option )
			{
				$settings[$key] = self::sanitise_settings($key,$option );
			}
		}
		return $settings;
	}

	/**
 	* Returns sanitised content
	 * @since    1.0.12
	 * @access   public
	*/
	public static function sanitise_settings( $key, $value )
	{
		$ret = null;
		switch( $key ){
			// text to bool conversion
			case 'is_on':
			case 'is_bottom':
			case 'showagain_tab':
			case 'wrap_shortcodes':
			case 'cookie_policy_link':
			case 'disable_logo':
			case 'is_cookie_policy_url':
			case 'is_personal_data_policy_url':
			case 'blocked_content_notify':
			case 'blocked_content_notify_auto_shutdown':
			case 'video_advanced_privacy':
			case 'maps_block':
			case 'captcha_block':
			case 'scanner_compatibility_mode':
			case 'enforce_youtube_privacy':
			case 'client_force_reinject_jquery':
			case 'display_dpo':
			case 'show_ntf_bar_on_not_yet_consent_choice':
			case 'with_css_effects':
				if ( $value == 'true' || $value === true )
				{
					$ret = true;
				}
				elseif ( $value == 'false' || $value === false )
				{
					$ret = false;
				}
				else
				{
					$ret = false;
				}
				break;
			//integer
			case 'scanner_start_hook_prio':
			case 'scanner_end_hook_prio':
			case 'blocked_content_notify_auto_shutdown_time':
				$ret = intval( $value );
				break;
			// hex colors
			case 'background':
			case 'text':
			case 'button_accept_link_color':
			case 'button_accept_button_color':
	        case 'button_reject_link_color':
			case 'button_reject_button_color':
	        case 'button_customize_link_color':
			case 'button_customize_button_color':
			case 'map_inline_notify_color':
			case 'map_inline_notify_background':


				if ( preg_match( '/^#[a-f0-9]{6}|#[a-f0-9]{3}$/i', $value ) )
				{
					$ret =  $value;
				}
				else {
					// Failover = assign '#000' (black)
					$ret =  '#000';
				}
				break;
			// html (no js code )
			case 'bar_heading_text':
			case 'website_name':
			case 'identity_name':
			case 'identity_address':
			case 'identity_vat_id':
			case 'notify_message':
			case 'showagain_text':
				$ret = wp_kses( $value, self::allowed_html_tags(), self::allowed_protocols() );
				break;
			case 'cookie_policy_url':
			case 'personal_data_policy_url':
			case 'license_code':
			case 'identity_email':
			case 'dpo_email':
				$ret = wp_kses( trim($value), self::allowed_html_tags(), self::allowed_protocols() );
				break;
			case 'custom_css':
				$ret = esc_html( $value );
				break;
			// Basic sanitisation for other fields
			default:
				$ret = sanitize_text_field( $value );
				break;
		}
		return $ret;
	}

	/**
	 * check for wp login page
	 * @since    1.3.5
	 * @access   public
	*/
	public static function is_wplogin()
	{
		if( function_exists('login_header') )
	    {
	    	return true;
	    }

		if (isset($_GET['page']) && $_GET['page'] == 'sign-in')
		{
		   return true;
		}

	    $ABSPATH_MY = str_replace(array('\\','/'), DIRECTORY_SEPARATOR, ABSPATH);
	    return ((in_array($ABSPATH_MY.'wp-login.php', get_included_files()) || in_array($ABSPATH_MY.'wp-register.php', get_included_files()) ) || (isset($_GLOBALS['pagenow']) && $GLOBALS['pagenow'] === 'wp-login.php') || $_SERVER['PHP_SELF']== '/wp-login.php');
	}


	/**
	 * check for buffer / script inclusion skip
	 * @since    1.3.5
	 * @access   public
	*/
	public static function check_buffer_skip_conditions()
	{
		$skip = 'false';

		global $pagenow;

		//widgets
		if( $pagenow && $pagenow === 'widgets.php' ) $skip = 'true';

		//amp
		if( ( function_exists( 'amp_is_request' ) && amp_is_request() ) ||
			isset( $_GET['amp'] ) ||
			strpos( $_SERVER['REQUEST_URI'], '/amp/' ) !== false ) $skip = 'true';

		//elementor
		if( isset( $_GET['elementor-preview'] ) ) $skip = 'true';

		//divi
		if ( isset( $_GET['et_fb'] ) && $_GET['et_fb'] == 1 ) $skip = 'true';

		//no admin
		if( is_admin() ) $skip = 'true';

		//no rss
		if( is_feed() ) $skip = 'true';

		//divi
		if( function_exists( 'et_fb_is_enabled' ) && et_fb_is_enabled() ) $skip = 'true';

		// page builder
		if( is_customize_preview() ) $skip = 'true';

		//xml rpc, ajax, admin
		if( ( defined( 'XMLRPC_REQUEST' ) && XMLRPC_REQUEST ) || isset($_POST['_wpnonce']) || (function_exists("wp_doing_ajax") && wp_doing_ajax()) || ( defined( 'DOING_AJAX' ) && DOING_AJAX ) || isset( $_SERVER["HTTP_X_REQUESTED_WITH"] ) )  $skip = 'true';

		//matomo
		if( strpos( $_SERVER['REQUEST_URI'], 'plugins/matomo/app' ) !== false ) $skip = 'true';

		//is_json
		if( ( function_exists('wp_is_json_request') && wp_is_json_request() ) || strpos( $_SERVER['REQUEST_URI'], '/wp-json/' ) !== false ) $skip = 'true';

		if( defined('REST_REQUEST') ) $skip = 'true';

		//wp-login and similar pages
		if( MyAgilePrivacy::is_wplogin() ) $skip = 'true';

		//rest request
        if (defined('REST_REQUEST') && REST_REQUEST // (#1)
                || isset($_GET['rest_route']) // (#2)
                        && strpos( $_GET['rest_route'], '/', 0 ) === 0)
                 $skip = 'true';

        //post
        if( !empty( $_POST ) ) $skip = 'true_due_to_post';

        return $skip;
	}


	/**
	 * Returns default settings
	 * @since    1.0.12
	 * @access   public
	 */
	public static function get_default_settings( $key='' )
	{
		$default_locale = get_locale();

		$settings = array(
			'is_on' 									=> 	true,
			'is_bottom'									=>	true,
			'bar_heading_text'							=>	'',
			'notify_message'							=> 	addslashes ( __('<div class="map-area-container"><div class="map_notification-message">This website uses technical and profiling cookies. Clicking on "Accept" authorises all profiling cookies. Clicking on "Refuse" or the X will refuse all profiling cookies. By clicking on "Customise" you can select which profiling cookies to activate.[myagileprivacy_extra_info]</div><div class="map_notification_container">[myagileprivacy_cookie_accept][myagileprivacy_cookie_reject][myagileprivacy_cookie_customize]</div></div>', 'myagileprivacy') ),
			'background' 								=> 	'#FFF',
			'text' 										=> 	'#333333',
			'button_accept_text'						=> 	__('Accept', 'myagileprivacy'),
			'button_accept_link_color' 					=> 	'#fff',
			'button_accept_button_color' 				=> 	'#34C659',
			'button_reject_text'						=> 	__('Refuse', 'myagileprivacy'),
			'button_reject_link_color' 					=> 	'#fff',
			'button_reject_button_color' 				=> 	'#636366',
			'button_customize_text'						=> 	__('Customize', 'myagileprivacy'),
			'button_customize_link_color' 				=> 	'#fff',
			'button_customize_button_color' 			=> 	'#8E8E93',


			'map_inline_notify_color'					=>	'#444',
			'map_inline_notify_background'				=>	'#FFF3CD',


			'website_name'								=>	'',
			'identity_name'								=>	'',
			'identity_address'							=>	'',
			'identity_vat_id'							=>	'',
			'identity_email'							=>	'',
			'license_code'								=>	'',
			'license_user_status'						=>	'Demo License',
			'license_valid'								=>	true,
			'grace_period'								=>	false,

			'customer_email'							=>	null,
			'summary_text'								=>	null,

			'notify_div_id' 							=> '#my-agile-privacy-notification-area',
			'showagain_tab' 							=> 	true,
			'wrap_shortcodes'							=>	false,
			'showagain_text'	 						=> 	__('Manage consent', 'myagileprivacy'),
			'notify_position_horizontal'				=> 	'right',
			'showagain_div_id' 							=> 	'my-agile-privacy-consent-again',


			'cookie_policy_link'						=>	false,
			'is_cookie_policy_url'						=>	false,
			'cookie_policy_url'							=>	null,
			'cookie_policy_page'						=> 	get_option('wp_page_for_privacy_policy') ? get_option('wp_page_for_privacy_policy') : 0,



			'is_personal_data_policy_url'				=>	false,
			'personal_data_policy_url'					=>	null,
			'personal_data_policy_page'					=>	0,


			'last_sync'									=>	null,
			'default_locale'							=>	$default_locale,
			'disable_logo'								=>	false,
			'wl'										=>	0,
			'pa'										=>	0,
			'last_legit_sync'							=>	null,
			'custom_css'								=>	null,
			'scan_mode'									=>	'turned_off',
			'blocked_content_notify'					=>	true,
			'blocked_content_notify_auto_shutdown'		=>	true,
			'blocked_content_notify_auto_shutdown_time'	=>	3000,

			'video_advanced_privacy'					=>	true,
			'maps_block'								=>	true,
			'captcha_block'								=>	true,
			'parse_config'								=>	null,
			'scanner_compatibility_mode'				=>	false,
			'scanner_hook_type'							=>	'init-shutdown',
			'scanner_start_hook_prio'					=>	-10000,
			'scanner_end_hook_prio'						=>	-10000,
			'scanner_log'								=>	null,

			'alt_accepted_all_cookie_name'				=>	null,
			'alt_accepted_something_cookie_name'		=>	null,

			'learning_mode_last_active_timestamp'		=>	null,
			'enforce_youtube_privacy'					=>	false,
			'client_force_reinject_jquery'				=>	false,
			'display_dpo'								=>	false,
			'dpo_email'									=>	null,
			'show_ntf_bar_on_not_yet_consent_choice'	=>	false,

			'with_css_effects'							=>	true,
		);

		$settings = apply_filters( 'map_plugin_settings', $settings);

		return $key!="" ? $settings[$key] : $settings;
	}

	/**
	 * Returns list of HTML tags allowed in HTML fields for use in declaration of wp_kset field validation.
	 * @since    1.0.12
	 * @access   public
	 */
	public static function allowed_html_tags()
	{
		$allowed_html = array(
			'a' => array(
				'href' => array(),
				'id' => array(),
				'class' => array(),
				'title' => array(),
				'target' => array(),
				'rel' => array(),
				'style' => array(),
				'role' => array(),
				'data-map_action' => array(),
			),
			'input' => array(
				'id' => array(),
				'name'=> array(),
				'type'=> array(),
				'value'=> array(),
				'class'=> array(),
				'data-cookie-baseindex'=>array(),
				'data-default-color'=>array(),
			),
			'b' => array(),
			'br' => array(
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'div' => array(
				'id' => array(),
				'class' => array(),
				'style' => array(),
				'data-nosnippet' => array(),
				'data-map_action'=> array(),
				'data-cookie-baseindex'=>array(),
				'data-cookie-name'=>array(),
			),
			'em' => array (
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'i' => array(),
			'img' => array(
				'src' => array(),
				'id' => array(),
				'class' => array(),
				'alt' => array(),
				'style' => array()
			),
			'p' => array (
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'span' => array(
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'strong' => array(
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'h1' => array(
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'h2' => array(
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'h3' => array(
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'h4' => array(
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'h5' => array(
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'h6' => array(
				'id' => array(),
				'class' => array(),
				'style' => array()
			),
			'label' => array(
				'id' => array(),
				'class' => array(),
				'style' => array(),
				'for' => array(),
				'data-map-enable' => array(),
				'data-map-disable' => array(),
			),
			'option' => array(
				'name' => array(),
				'value' => array(),
				'selected' => array(),
			),
		);
		$html5_tags=array('article','section','aside','details','figcaption','figure','footer','header','main','mark','nav','summary','time');
		foreach($html5_tags as $html5_tag)
		{
			$allowed_html[$html5_tag]=array(
				'id' => array(),
				'class' => array(),
				'style' => array()
			);
		}
		return $allowed_html;
	}


	/**
	 * Returns list of allowed protocols, used in wp_kset field validation.
	 * @since    1.0.12
	 * @access   public
	 */
	public static function allowed_protocols()
	{
		return array ('http', 'https');
	}


	/**
	 * Returns JSON object containing user settings
	 * @since    1.0.12
	 * @access   public
	 */
	public static function get_json_settings()
	{
	  $settings = self::get_settings();

	  $return_settings = array(
	  	'notify_div_id'								=> 	$settings['notify_div_id'],
	  	'showagain_tab'								=> 	$settings['showagain_tab'],
	  	'notify_position_horizontal'				=> 	$settings['notify_position_horizontal'],
	  	'showagain_div_id'							=> 	$settings['showagain_div_id'],
		'blocked_content_text'						=>	__('Warning: some page functionalities could not work due to your privacy choices', 'myagileprivacy'),
		'inline_notify_color'						=>	$settings['map_inline_notify_color'],
		'inline_notify_background'					=>	$settings['map_inline_notify_background'],
		'blocked_content_notify_auto_shutdown_time'	=>	$settings['blocked_content_notify_auto_shutdown_time'],

		'scan_mode'									=>	$settings['scan_mode'],
		'cookie_reset_timestamp'					=>	( isset( $settings['cookie_reset_timestamp'] ) ) ? '_'.$settings['cookie_reset_timestamp'] : null,
		'show_ntf_bar_on_not_yet_consent_choice'	=>	$settings['show_ntf_bar_on_not_yet_consent_choice'],

	  );

	  return $return_settings;
	}


	/**
	 * Makes a call to the WP License Manager API.
	 *
	 * @param $params   array   The parameters for the API call
	 * @return          array   The API response
	 * @since    1.0.12
	 * @access   public
	 */
	public static function call_api( $params )
	{
	    $url = MAP_API_ENDPOINT;

	    // Append parameters for GET request
	    $url .= '?' . http_build_query( $params );

	    // Send the request
	    $response = wp_remote_get( $url, array( 'sslverify' => false ) );
	    if ( is_wp_error( $response ) ) {
	        return false;
	    }

	    $response_body = wp_remote_retrieve_body( $response );
	    $result = json_decode( $response_body, true );

	    return $result;
	}

	/**
	 * write to log file
	 * @since    1.0.12
	 * @access   public
	 */
    public static function write_log($log)
    {
    	$filePath = '/app/debug.txt';

    	$bt = debug_backtrace();

    	$depth = 0;

        $file = isset($bt[$depth])     ? $bt[$depth]['file'] : null;
        $line = isset($bt[$depth])     ? $bt[$depth]['line'] : 0;
        $func = isset($bt[$depth + 1]) ? $bt[$depth + 1]['function'] : null;

        if (is_array($log) || is_object($log)) {
            $data = print_r($log, true);
        } else {
            $data = $log;
        }

        $string = "file=$file, line=$line, func=$func: ".$data."\n";

    	file_put_contents( $filePath, $string, FILE_APPEND );
    }
}